#include <math.h>

void polint(double *xa, double *ya, int n, double x, double *y, double *dy)
{
   int i,m, ns=1;
   double den, dif, dift, ho, hp, w;
   double *c, *d;
   
   dif = fabs(x - xa[0]);
   c = (double *) malloc((size_t) n*sizeof(double));
   d = (double *) malloc((size_t) n*sizeof(double));
   for(i=0; i<n; i++)
   {
      dift=fabs(x-xa[i]);
      if(dift < dif)
      {
         ns = i;
         dif = dift;
      }
      c[i] = ya[i];
      d[i] = ya[i];
   }
   (*y) = ya[ns--];
   for(m=1; m<n; m++)
   {
      for(i=0; i<n-m; i++)
      {
         ho = xa[i] - x;
         hp = xa[i+m] - x;
         w = c[i+1] - d[i];
         den = ho - hp;
         if(den == 0.0)
         {
            printf("error in polynomial interpolation\n");
            free(d);
            free(c);
            exit(0);
         }
         den = w/den;
         d[i] = hp * den;
         c[i] = ho * den;
      }
      (*dy) = (2.0 * ns < (n - m)) ? c[ns+1] : d[ns--];
      (*y) += (*dy);
   }
   free(d);
   free(c);
}      
            
 
double trapzd(double (*func)(double, double*), double *extra_par, double a, double b, int n)
{
   double x, tnm, sum, del;
   static double s;
   int it, j;
   
   if(n == 1)
   {
      s = 0.5 * (b - a) * ( (*func)(a, extra_par) + (*func)(b, extra_par) );
      return (s);                  
   }
   else
   {
      for(it=1, j=0; j<n-2; j++)
         it <<= 1;
      tnm = it;
      del = (b - a) / tnm;
      x = a + 0.5 * del;
      for(sum=0.0, j=0; j<it; j++, x+=del)
         sum += (*func)(x, extra_par);
      s = 0.5 * (s + (b - a) * sum / tnm);
      return (s);
   }
}           

#define JMAX 20
#define JMAXP (JMAX+1)
double integrate(double (*func)(double, double*), double *extra_par, double a, double b)
{
   void polint(double *xa, double *ya, int n, double x, double *y, double *dy);
   double trapzd(double (*func)(double, double*), double *extra_par, double a, double b, int n);
   double ss, dss;
   double s[JMAX], h[JMAXP];
   int j;
   int k=5;
   double eps=1e-6;
   
   h[0] = 1.0;
   for(j=0; j<JMAX; j++)
   {
      s[j] = trapzd(func, extra_par, a, b, j);
      if(j >= k - 1)
      {
         polint(&h[j - (k - 1)], &s[j - (k - 1)], k, 0.0, &ss, &dss);
         if(fabs(dss) <= eps * fabs(ss))  
         {
            return(ss);
         } 
      }
      h[j+1] = 0.25 * h[j];
   }
   printf("Too many steps for integration\n");
   return(0.0);
}         

/***********************************************************
 Adaptive Gaussian quadrature
 ***********************************************************/
 
#define uflow 	1e-200
#define oflow 	1e100
#define epmach 	1e-200
#define LIMIT 	500
#define MAXP1 	21
#define Pi      M_PI
#define COSINE 	1
#define SINE	2

#ifndef FALSE
#define FALSE   0
#endif
#ifndef TRUE
#define TRUE    1
#endif
#ifndef min
#define min(a,b)    (((a) < (b)) ? (a) : (b))
#endif
#ifndef max
#define max(a,b)    (((a) > (b)) ? (a) : (b))
#endif
 
double dqext(int *n,double epstab[],double *abserr,
	double res3la[],int *nres)
{
	double delta1,delta2,delta3,dmax1,epsinf;
	double error,err1,err2,err3,e0,e1,e1abs,e2,e3;
	double res,result,ss,tol1,tol2,tol3;
	int NN,i,ib,ib2,ie,indx,k1,k2,k3,limexp,newelm,num;
	
	(*nres)++;
	NN = *n;
    NN++;   /* make NN a FORTRAN array index */
	*abserr = oflow;
    result = epstab[*n];
	if (NN < 3) goto _100;		/* N < 3 */
	limexp = 50;			/* limexp = 50 */
    epstab[*n+2] = epstab[*n];
    newelm = (*n)/2;      /* (n-1)/2 */
    epstab[*n] = oflow;
	num = NN;
	k1 = NN;
	for (i = 1; i <= newelm; i++) 
    {
		k2 = k1 - 1;
		k3 = k1 - 2;
		res = epstab[k1+1];
		e0 = epstab[k3-1];
		e1 = epstab[k2-1];
		e2 = res;
        e1abs = fabs(e1);
		delta2 = e2 - e1;
        err2 = fabs(delta2);
        tol2 = max(fabs(e2),e1abs) * epmach;
		delta3 = e1 - e0;
        err3 = fabs(delta3);
        tol3 = max(e1abs,fabs(e0)) * epmach;
		if ((err2 > tol2) || (err3 > tol3)) goto _10;
		result = res;
		*abserr = err2 + err3;
		goto _100;
        
        _10:
		e3 = epstab[k1-1];
		epstab[k1-1] = e1;
		delta1 = e1 - e3;
        err1 = fabs(delta1);
        tol1 = max(e1abs,fabs(e3)) * epmach;
		if ((err1 <= tol1) || (err2 <= tol2) || (err3 <= tol3)) goto _20;
		ss = 1.0/delta1 + 1.0/delta2 - 1.0/delta3;
        epsinf = fabs(ss*e1);
		if (epsinf > 1.0e-4) goto _30;
        
        _20:
        NN = i + i - 1;
		goto _50;
        
        _30:
		res = e1 + 1.0 / ss;
		epstab[k1-1] = res; 
		k1 -= 2;
        error = err2 + fabs(res - e2) + err3;
		/*if (error > (*abserr)) goto _40;*/
		if (error > (*abserr)) continue;
		*abserr = error;
		result = res;
        /*_40:*/
	}
    _50:
	if (NN == limexp) NN = 2 * (limexp/2) - 1;
	ib = 1;						/* ib = 1 */
	if (((num/2) * 2 ) == num) ib = 2;		/* ib = 2 */
	ie = newelm + 1;
	for (i = 1;i <= ie; i++) {
		ib2 = ib + 2;
		epstab[ib-1] = epstab[ib2-1];
		ib = ib2;
	}
	if (num == NN) goto _80;
	indx = num - NN + 1;
    for (i = 1;i <= NN; i++) {
		epstab[i-1] = epstab[indx-1];
		indx++;
	}
    
    _80:
    if (*nres > 3) goto _90;       /* nres >= 4 */
    res3la[(*nres)-1] = result;
	*abserr = oflow;
	goto _100;

    _90:
    *abserr = fabs(result - res3la[2]) + fabs(result - res3la[1]) +
        fabs(result - res3la[0]);
	res3la[0] = res3la[1];
	res3la[1] = res3la[2];
	res3la[2] = result;
    
    _100:
    *abserr = max(*abserr,5.0 * epmach * fabs(result));
	*n = NN - 1;
	return result;
	}


void dqsort(int limit,int last,int *maxerr,double *ermax,double elist[],
	int iord[],int *nrmax)
{
	double errmax,errmin;
	int i,ibeg,ido,isucc,j,jbnd,jupbn,k;

	if (last > 1) goto _10;
	iord[0] = 0;
	iord[1] = 1;
	goto _90;

        _10:
	errmax = elist[*maxerr];
	if (*nrmax == 0) goto _30;
	ido = (*nrmax) - 1;
	for (i = 0;i <= ido; i++) {
		isucc = iord[*nrmax-1];
		if (errmax <= elist[isucc]) goto _30;
		iord[*nrmax] = isucc;
		(*nrmax)--;
	}

        _30:
	jupbn = last;
	if (last > (limit/2 + 2))
		jupbn = limit + 3 - last;
	errmin = elist[last];
	jbnd = jupbn - 1;
	ibeg = *nrmax + 1;
	if (ibeg > jbnd) goto _50;
	for (i = ibeg; i <= jbnd; i++) {
		isucc = iord[i];
		if (errmax >= elist[isucc]) goto _60;
		iord[i-1] = isucc;
	}

        _50: 
	iord[jbnd] = *maxerr;
	iord[jupbn] = last;
	goto _90;

        _60:
	iord[i-1] = *maxerr;
	k = jbnd;
	for (j = i;j <= jbnd; j++) {
		isucc = iord[k];
		if (errmin < elist[isucc]) goto _80;
		iord[k+1] = isucc;
		k--;
	}
	iord[i] = last;
	goto _90;

        _80:
	iord[k+1] = last;

        _90:
	*maxerr = iord[*nrmax];
	*ermax = elist[*maxerr];
	return;  
}	

 
double G_K21(double f(double, double *), double *extra_par, double a,double b,double *abserr,
	double *resabs,double *resasc)
{
	static long double XGK21[11] = {
		0.99565716302580808074,
		0.97390652851717172008,
		0.93015749135570822600,
		0.86506336668898451073,
		0.78081772658641689706,
		0.67940956829902440623,
		0.56275713466860468334,
		0.43339539412924719080,
		0.29439286270146019813,
		0.14887433898163121088,
		0.00000000000000000000};
	static long double WGK21[11] = {
		0.01169463886737187428,
		0.03255816230796472748,
		0.05475589657435199603,
		0.07503967481091995277,
		0.09312545458369760554,
		0.10938715880229764190,
		0.12349197626206585108,
		0.13470921731147332593,
		0.14277593857706008080,
		0.14773910490133849137,
		0.14944555400291690566};
	static long double WG10[5] = {
		0.06667134430868813759,
		0.14945134915058059315,
		0.21908636251598204400,
		0.26926671930999635509,
		0.29552422471475287017};
	double fv1[10],fv2[10];
	double absc,centr,dhlgth,dmax1,dmin1;
	double fc,fsum,fval1,fval2,hlgth;
	double resg,resk,reskh,result;
	int j,jtw,jtwm1;
	
	centr = 0.5 * (a + b);
	hlgth = 0.5 * (b - a);
        dhlgth = fabs(hlgth);

	resg = 0.0;
	fc=(*f)(centr, extra_par);
	resk = fc * WGK21[10];
        *resabs = fabs(resk);
	for (j = 0; j < 5; j++) 
	{
		jtw = 2 * j + 1;
		absc = hlgth * XGK21[jtw];
		fval1 = (*f)(centr-absc, extra_par);
		fval2 = (*f)(centr+absc, extra_par);
		fv1[jtw] = fval1;
		fv2[jtw] = fval2;
		fsum = fval1 + fval2;
		resg += WG10[j] * fsum;
		resk += WGK21[jtw] * fsum;
                *resabs = *resabs + WGK21[jtw] * (fabs(fval1) + fabs(fval2));
	}
	for (j = 0; j < 5; j++) 
	{
		jtwm1 = j * 2;
		absc = hlgth * XGK21[jtwm1];
		fval1 = (*f)(centr-absc, extra_par);
		fval2 = (*f)(centr+absc, extra_par);
		fv1[jtwm1] = fval1;
		fv2[jtwm1] = fval2;
		fsum = fval1 + fval2;
		resk = resk + WGK21[jtwm1] * fsum;
                *resabs = (*resabs) + WGK21[jtwm1] * (fabs(fval1) + fabs(fval2));
	}
	reskh = resk * 0.5;
        *resasc = WGK21[10] * fabs(fc - reskh);
	for (j = 0; j < 10; j++ )
        *resasc = (*resasc) + WGK21[j] * (fabs(fv1[j] - reskh) + fabs(fv2[j] - reskh));
	result = resk * hlgth;
	*resabs = (*resabs) * dhlgth;
	*resasc = (*resasc) * dhlgth;
        *abserr = fabs((resk - resg) * hlgth);

	if ((*resasc != 0.0) && (*abserr != 0.0))
        *abserr = (*resasc) * min(1.0,pow((200.0 * (*abserr)/(*resasc)),1.5));
	if (*resabs > uflow/(50.0 * epmach))
		*abserr = max(epmach * 50.0 * (*resabs),(*abserr)); 	
	return result;
}

/*****************************************************************************
!
!! QAGS estimates the integral of a function.
!
!  Discussion:
!
!    The routine calculates an approximation RESULT to a definite integral   
!      I = integral of F over (A,B),
!    hopefully satisfying
!      || I - RESULT || <= max ( EPSABS, EPSREL * ||I|| ).
!
!  Author:
!
!    Robert Piessens, Elise de Doncker-Kapenger, 
!    Christian Ueberhuber, David Kahaner
!
!  Reference:
!
!    Robert Piessens, Elise de Doncker-Kapenger, 
!    Christian Ueberhuber, David Kahaner,
!    QUADPACK, a Subroutine Package for Automatic Integration,
!    Springer Verlag, 1983
!
!  Parameters:
!
!    Input, external real F, the name of the function routine, of the form
!      function f ( x )
!      real f
!      real x
!    which evaluates the integrand function.
!
!    Input, real A, B, the limits of integration.
!
!    Input, real EPSABS, EPSREL, the absolute and relative accuracy requested.
!
!    Output, real RESULT, the estimated value of the integral.
!
!    Output, real ABSERR, an estimate of || I - RESULT ||.
!
!    Output, integer NEVAL, the number of times the integral was evaluated.
!
!    Output, integer IER, error flag.
!                     ier = 0 normal and reliable termination of the
!                             routine. it is assumed that the requested
!                             accuracy has been achieved.
!                     ier > 0 abnormal termination of the routine
!                             the estimates for integral and error are
!                             less reliable. it is assumed that the
!                             requested accuracy has not been achieved.
!                         = 1 maximum number of subdivisions allowed
!                             has been achieved. one can allow more sub-
!                             divisions by increasing the data value of
!                             limit in qags (and taking the according
!                             dimension adjustments into account).
!                             however, if this yields no improvement
!                             it is advised to analyze the integrand
!                             in order to determine the integration
!                             difficulties. if the position of a
!                             local difficulty can be determined (e.g.
!                             singularity, discontinuity within the
!                             interval) one will probably gain from
!                             splitting up the interval at this point
!                             and calling the integrator on the sub-
!                             ranges. if possible, an appropriate
!                             special-purpose integrator should be used,
!                             which is designed for handling the type
!                             of difficulty involved.
!                         = 2 the occurrence of roundoff error is detec-
!                             ted, which prevents the requested
!                             tolerance from being achieved.
!                             the error may be under-estimated.
!                         = 3 extremely bad integrand behavior occurs
!                             at some  points of the integration
!                             interval.
!                         = 4 the algorithm does not converge. roundoff
!                             error is detected in the extrapolation
!                             table. it is presumed that the requested
!                             tolerance cannot be achieved, and that the
!                             returned result is the best which can be
!                             obtained.
!                         = 5 the integral is probably divergent, or
!                             slowly convergent. it must be noted that
!                             divergence can occur with any other value
!                             of ier.
!                         = 6 the input is invalid, because
!                             epsabs < 0 and epsrel < 0,
!                             result, abserr and neval are set to zero.
!
!  Local Parameters:
!
!           alist     - list of left end points of all subintervals
!                       considered up to now
!           blist     - list of right end points of all subintervals
!                       considered up to now
!           rlist(i)  - approximation to the integral over
!                       (alist(i),blist(i))
!           rlist2    - array of dimension at least limexp+2 containing
!                       the part of the epsilon table which is still
!                       needed for further computations
!           elist(i)  - error estimate applying to rlist(i)
!           maxerr    - pointer to the interval with largest error
!                       estimate
!           errmax    - elist(maxerr)
!           erlast    - error on the interval currently subdivided
!                       (before that subdivision has taken place)
!           area      - sum of the integrals over the subintervals
!           errsum    - sum of the errors over the subintervals
!           errbnd    - requested accuracy max(epsabs,epsrel*
!                       abs(result))
!           *****1    - variable for the left interval
!           *****2    - variable for the right interval
!           last      - index for subdivision
!           nres      - number of calls to the extrapolation routine
!           numrl2    - number of elements currently in rlist2. if an
!                       appropriate approximation to the compounded
!                       integral has been obtained it is put in
!                       rlist2(numrl2) after numrl2 has been increased
!                       by one.
!           small     - length of the smallest interval considered
!                       up to now, multiplied by 1.5
!           erlarg    - sum of the errors over the intervals larger
!                       than the smallest interval considered up to now
!           extrap    - logical variable denoting that the routine is
!                       attempting to perform extrapolation i.e. before
!                       subdividing the smallest interval we try to
!                       decrease the value of erlarg.
!           noext     - logical variable denoting that extrapolation
!                       is no longer allowed (true value)
************************************************************************/
double dqags(double f(double, double *), double *extra_par, double a,double b,double epsabs,
	double epsrel,double *abserr,int *neval,int *ier)
{
	double abseps,alist[LIMIT],area,area1,area12,area2;
	double a1,a2,blist[LIMIT],b1,b2,correc,defabs,defab1;
	double defab2,dres,elist[LIMIT],erlarg,erlast,errbnd;
	double errmax,error1,error2,erro12,errsum,ertest;
	double resabs,reseps,result,res3la[3],rlist[LIMIT];
	double rlist2[52],small;

	int id,ierro,iord[LIMIT],iroff1,iroff2,iroff3,jupbnd,k,ksgn;
	int ktmin,last,maxerr,nres,nrmax,numrl2;
        int limit;
	int extrap,noext;

        limit = LIMIT -1;
        /* Test validity of parameters. */	
	*ier = 0;
	*neval = 0;
	result = 0.0;
	*abserr = 0.0;
	alist[0] = a;
	blist[0] = b;
	rlist[0] = 0.0;
	elist[0] = 0.0;
	if ((epsabs < 0.0) && (epsrel < 0.0)) *ier = 6;
	if (*ier == 6) return result;

        /* First approximation to the integral. */
	ierro = 0;
        result = G_K21(f, extra_par,a,b,abserr,&defabs,&resabs);
        
        /* Test on accuracy. */
        dres = fabs(result);
	errbnd = max(epsabs,epsrel*dres);
        last = 1;
	rlist[0] = result;
	elist[0] = *abserr;	
	iord[0] = 0;
	if ((*abserr <= 100.0 * epmach * defabs) && (*abserr > errbnd))
		*ier = 2;
	if (limit == 0) *ier = 1;
	if ((*ier != 0) || (*abserr <= errbnd) && (*abserr != resabs) ||
		(*abserr == 0.0)) 
        {
		goto _140;
	}	
        
        /* Initialization. */
	rlist2[0] = result;
	errmax = *abserr;
	maxerr = 0; 			/* maxerr = 1 */
	area = result;
	errsum = *abserr;
	*abserr = oflow;
	nrmax = 0;
        nres = 0;          /* nres = 0 */
	numrl2 = 1;			/* numrl2 = 2 */
	ktmin = 0;
	extrap = FALSE;
	noext = FALSE;
	ierro = 0;
	iroff1 = 0;
	iroff2 = 0;
	iroff3 = 0;
	ksgn = -1;
	if (dres > (1.0 - 50.0 * epmach) * defabs)
		ksgn = 1;

        /* Main loop. */
	for (last = 1; last <= limit; last++) 
        {

                /* Bisect the interval with the nrmax-th largest error estimate. */
		a1 = alist[maxerr];
		b1 = 0.5 * (alist[maxerr] + blist[maxerr]);
		a2 = b1;
		b2 = blist[maxerr];
		erlast = errmax;
		area1 = G_K21(f, extra_par, a1,b1,&error1,&resabs,&defab1);
		area2 = G_K21(f, extra_par, a2,b2,&error2,&resabs,&defab2);
                
                /* Improve previous approxminations to integral and error
                   and test for accuracy. */
		area12 = area1 + area2;
		erro12 = error1 + error2;
		errsum = errsum + erro12 - errmax;
		area = area + area12 - rlist[maxerr];
		if ((defab1 == error1) || (defab2 == error2)) goto _15; 
                if ((fabs(rlist[maxerr] - area12) > 1.0e-5 * fabs(area12))
			|| (erro12 < .99 * errmax)) goto _10;
		if (extrap) iroff2++;
		else iroff1++;
        
                _10:
		if ((last > 9) && (erro12 > errmax))    /* last > 10 */
			iroff3++;
        
                _15:
		rlist[maxerr] = area1;
		rlist[last] = area2;
                errbnd = max(epsabs,epsrel * fabs(area));
		
                /* Test for roundoff error and eventually set error flag. */
		if (((iroff1 + iroff2) >= 10) || (iroff3 >= 20))
			*ier = 2;
		if (iroff2 > 5)
			*ier = 3;

                /* Set error flag in the case that the number of subintervals
                equals limit. */
		if (last == limit)	/* last == limit */
			*ier = 1;
					
                /* Set error flag in the case of bad integrand behavior at some
                 points in the integration range. */
                if (max(fabs(a1),fabs(b2)) <= (1.0 +1000.0 * epmach) * (fabs(a2) + 1000.0*uflow)) 
			*ier = 4;

                /* Append the newly-created intervals to the list. */
		if (error2 > error1) goto _20;
		alist[last] = a2;
		blist[maxerr] = b1;
		blist[last] = b2;
		elist[maxerr] = error1;
		elist[last] = error2;
		goto _30;
        
                _20:
		alist[maxerr] = a2;
		alist[last] = a1;
		blist[last] = b1;
		rlist[maxerr] = area2;
		rlist[last] = area1;
		elist[maxerr] = error2;
		elist[last] = error1;

                /* Call dqsort to maintain the descending ordering in the list of error
                   estimates and select the subinterval with nrmax-th largest
                   error estimate (to be bisected next). */
        
                _30:
		dqsort(limit,last,&maxerr,&errmax,elist,iord,&nrmax);
		if (errsum <= errbnd) goto _115;
		if (*ier != 0) goto _100;
		if (last == 1) goto _80;	/* last == 2 */
		if (noext) continue;		/* goto 90 */
		erlarg -= erlast;
                if (fabs(b1-a1) > small)
			erlarg += erro12;
		if (extrap) goto _40;
		
                /* Test whether the interval to be bisected next is the smallest interval. */
                if ((fabs(blist[maxerr] - alist[maxerr])) > small) 
			/*goto _90;*/
			continue;
		extrap = TRUE;
		nrmax = 1;		/* nrmax = 2 */		
        
                _40:
		if ((ierro == 3) || (erlarg <= ertest)) goto _60;

                /* The smallest interval has the largest error. Before bisecting, decrease
                   the sum of the erors over the larger intervals (erlarg) and
                   perform extrapolation.) */
		id = nrmax;
		jupbnd = last;
		if (last > (2 + limit/2))
			jupbnd = limit + 3 - last;
		for (k = id;k <= jupbnd; k++) 
                {
			maxerr = iord[nrmax];
			errmax = elist[maxerr];
                        if (fabs(blist[maxerr] - alist[maxerr]) > small)
				/*goto _90;	*/
				continue;
			nrmax++;
		}

                /* Perform extrapolation. */
                _60:
		numrl2++;
		rlist2[numrl2] = area;
		reseps=dqext(&numrl2,rlist2,&abseps,res3la,&nres);
		ktmin++;
		if ((ktmin > 5) && (*abserr < 1.0e-3 * errsum)) *ier = 5;
		if (abseps >= *abserr) goto _70;
		ktmin = 0;
		*abserr = abseps;
		result = reseps;
		correc = erlarg;
                ertest = max(epsabs,epsrel * fabs(reseps));
		if (*abserr <= ertest) goto _100;

                /* Prepare bisection of the smallest interval. */
                _70:
		if (numrl2 == 0) noext = TRUE;
		if (*ier == 5) goto _100;
		maxerr = iord[0];
		errmax = elist[maxerr];
		nrmax = 0;
		extrap = FALSE;
		small = small * 0.5;
		erlarg = errsum;
		/*goto _90;*/
		continue;
        
                _80:
                small = fabs(b-a)*0.375;
		erlarg = errsum;
		ertest = errbnd;
		rlist2[1] = area;

                /*_90:*/
	}
    					/* 90: */
        _100:
	if (*abserr == oflow) goto _115;
	if ((*ier + ierro) == 0) goto _110;
	if (ierro == 3) *abserr += correc;
	if (*ier == 0) *ier = 3;
	if ((result != 0.0) && (area != 0.0)) goto _105;
	if (*abserr > errsum) goto _115;
	if (area == 0.0) goto _130;
	goto _110;

        _105:
        if (*abserr/fabs(result) > errsum/fabs(area)) goto _115;

        /* Test on divergence. */
        _110:
        if ((ksgn == -1) && (max(fabs(result),fabs(area)) <= defabs * .01))
		goto _130;
	if ((0.01 > result/area) || (result/area > 100.0) ||
        (errsum > fabs(area))) *ier = 6;
	goto _130;

        /* Compute global integral. */
        _115:
	result = 0.0;
	for (k = 0; k <= last; k++)
		result += rlist[k];
	*abserr = errsum;

        _130:
	if (*ier > 2) (*ier)--;

        _140:
        *neval = 42 * last - 21;
    
	return result;	
}	
