

// CodeCogs GNU General Public License Agreement
// Copyright (C) 2004-2005 CodeCogs, Zyba Ltd, Broadwood, Holford, TA5 1DU, England.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by CodeCogs. 
// You must retain a copy of this licence in all copies. 
//
// This program is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the Adapted GNU General Public License for more details.
//
// *** THIS SOFTWARE CAN NOT BE USED FOR COMMERCIAL GAIN. ***
// --------------------------------------------------------------------------------- 
 

double gammaUpper_reg_inv(double a, double y0)
{
  if(y0<=0 || a<=0)
  {
    cc_error("gammaUpper_reg_inv DOMAIN");
    return 0;
  }
 
  int dir;
 
  /* bound the solution */
  double x0 = MAXNUM;
  double yl = 0;
  double x1 = 0;
  double yh = 1.0;
  double dithresh = 5.0 * MACHEP;
 
  /* approximation to inverse function */
  double d = 1.0/(9.0*a);
  double y = ( 1.0 - d - Stats::Dists::Continuous::StdNormal::CDF_inv(y0) *
sqrt(d) );
  double x = a * y * y * y;
 
  double lgm = Maths::Special::Gamma::log_gamma(a);
 
  for(int i=0; i<10; i++ )
	{
    if( x > x0 || x < x1 )
      goto ihalve;
    y = Maths::Special::Gamma::gammaUpper_reg(a,x);
    if( y < yl || y > yh )
      goto ihalve;
    if( y < y0 )
		{
      x0 = x;
      yl = y;
		}
    else
		{
      x1 = x;
      yh = y;
		}
    /* compute the derivative of the function at this point */
    d = (a - 1.0) * log(x) - x - lgm;
    if( d < -MAXLOG )
      goto ihalve;
    d = -exp(d);
    /* compute the step to the next approximation of x */
    d = (y - y0)/d;
    if( fabs(d/x) < MACHEP )
      goto done;
    x = x - d;
	}
 
  /* Resort to interval halving if Newton iteration did not converge. */
ihalve:
 
    d = 0.0625;
  if( x0 == MAXNUM )
	{
    if( x <= 0.0 )
      x = 1.0;
    while( x0 == MAXNUM )
		{
      x = (1.0 + d) * x;
      y = Maths::Special::Gamma::gammaUpper_reg( a, x );
      if( y < y0 )
			{
        x0 = x;
        yl = y;
        break;
			}
      d = d + d;
		}
	}
  d = 0.5;
  dir = 0;
 
  for(int i=0; i<400; i++ )
	{
    x = x1  +  d * (x0 - x1);
    y = Maths::Special::Gamma::gammaUpper_reg( a, x );
    lgm = (x0 - x1)/(x1 + x0);
    if( fabs(lgm) < dithresh )
      break;
    lgm = (y - y0)/y0;
    if( fabs(lgm) < dithresh )
      break;
    if( x <= 0.0 )
      break;
    if( y >= y0 )
		{
      x1 = x;
      yh = y;
      if( dir < 0 )
			{
        dir = 0;
        d = 0.5;
			}
      else if( dir > 1 )
        d = 0.5 * d + 0.5;
      else
        d = (y0 - yl)/(yh - yl);
      dir += 1;
		}
    else
		{
      x0 = x;
      yl = y;
      if( dir > 0 )
			{
        dir = 0;
        d = 0.5;
			}
      else if( dir < -1 )
        d = 0.5 * d;
      else
        d = (y0 - yl)/(yh - yl);
      dir -= 1;
		}
	}
#ifndef NDEBUG
  if( x == 0.0 )
    cc_error( "gamma_inv UNDERFLOW" );
#endif
 
done:
    return( x );
}
