#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>


#define bipow(x,y) (((y)==0)? 1:(x))
#define max(x, y) ((x)>(y)?(x):(y))
#define min(x, y) ((x)<(y)?(x):(y))
#define logit(x) ( log( (x) / ( 1.0 - (x) ) ) )
#define inv_logit(x) ( 1.0 / ( 1.0 + exp(-(x)) ) )

#define MISSING 99999
#define INFINITY 10000
#define close_to_0 1e-20
#define close_to_1 0.99999
#define close_to_05 0.49999
#define num_ini_estimation 50
#define num_ini_test 10

#include "matrix.h"
#include "mathfunc.h"
#include "distribution.h"
#include "statistics.h"
#include "optimization.h"

#include "datastruct.h" /*specifying data structure*/

PEOPLE *people; /* pointer to people */
int p_size;

COMMUNITY *community;
int n_community;

CFG_PARS cfg_pars;
int DEBUG = 0;

int max_epi_duration;
long seed;
double *pdf_incubation, *sdf_incubation;

int size_sample_status;
double *importance_weight;

int ITER;
int serial_number;
int n_need_MCEM;
int n_need_OEM;

#include "config.h" /*read in configuration parameters for flu and trial*/
#include "utility.h"
#include "simulate.h"
#include "preamble.h"
#include "derivatives.h"
#include "estimation.h"

/* Function getsize, below, determines the number of people in the
   population by counting the number of lines in the population input
   file. This function is used once at the start to assign the p_size
   variable. This will always be in increments of 2000 people*/

int get_size(FILE *in)
{
  int c, i=0;
  rewind(in);
  while((c=getc(in))!=EOF)
    if(c == '\n') i++;
  return i;
 }



/* The main routine starts here.*/

int main(int argc, char *argv[])
{
  FILE *file;    
  PEOPLE *person, *member;
  int h, i, j, k, l, n, m, r, t;
  int n_iter;
  int ignore, found, contact_mode; 
  int estimation_error_type, test_error_type;
  int n_par, n_b_mode, n_p_mode, n_u_mode,  n_q_mode, n_c2p_covariate, n_p2p_covariate;
  int n_pat_covariate, n_imm_covariate;
  int n_covariate, n_time_ind_covariate, n_time_dep_covariate;
  int n_sus_p2p_covariate, n_inf_p2p_covariate, n_int_p2p_covariate;                     
  int n_record, possible_pre_immune, possible_escape;
  int possible_sym, possible_sym_start, possible_sym_stop;
  int possible_asym, possible_asym_start, possible_asym_stop;
  int start_day, stop_day;
  int n_imm, n_sym, n_asym, n_esc;
  char file_name[50];

  double CPI, se_CPI, SAR, se_SAR, se_logit_SAR, R0, se_R0, lower, upper;
  double pr, mean_sym, infective_prob;
  double s, log_likelihood, pvalue, offset;
  double *est, *der, *value;
  double p_esc, log_p_esc_der;

  CONTACT *ptr_contact, *ptr2_contact;
  STATUS *ptr_stat, *ptr2_stat;
  RISK *ptr_risk, *ptr2_risk;
  RISK_CLASS *ptr_class, *ptr2_class;
  INTEGER_CHAIN *ptr_integer, *ptr2_integer;

  MATRIX var, var_logit;
  time_t t1, t2;

  t1 = time(NULL);
  if(argc > 1)
     seed = atoi(argv[1]);
  else
     seed = 12345678;
  mt_init(seed);
  
  if(argc > 2)
     serial_number = atoi(argv[2]);
  
  cfg_pars = get_cfg_pars();
  // Do some initialization
  n_b_mode = cfg_pars.n_b_mode;//number of c2p modes
  n_p_mode = cfg_pars.n_p_mode;//number of p2p modes
  n_u_mode = cfg_pars.n_u_mode;//number of pre-immunity groups
  n_q_mode = cfg_pars.n_q_mode;//number of pre-immunity groups
  n_c2p_covariate = cfg_pars.n_c2p_covariate;//number of c2p covariates
  n_sus_p2p_covariate = cfg_pars.n_sus_p2p_covariate;//number of susceptible covariates for p2p
  n_inf_p2p_covariate = cfg_pars.n_inf_p2p_covariate;//number of infective covariates for p2p
  n_int_p2p_covariate = cfg_pars.n_int_p2p_covariate;//number of interaction terms for p2p 
  n_p2p_covariate = cfg_pars.n_p2p_covariate;//total number of covariates for p2p
  n_imm_covariate = cfg_pars.n_imm_covariate;//total number of covariates for pre-immunity
  n_pat_covariate = cfg_pars.n_pat_covariate;//total number of covariates for pre-immunity
  n_par = cfg_pars.n_par;//total number of parameters
  n_time_ind_covariate = cfg_pars.n_time_ind_covariate;//number of time-independent covariates
  n_time_dep_covariate = cfg_pars.n_time_dep_covariate;//number of time-dependent covariates
  

  importance_weight = NULL;
  pdf_incubation = sdf_incubation = NULL;

  l = cfg_pars.max_incubation - cfg_pars.min_incubation + 1;
  pdf_incubation = (double *)malloc((size_t) (l * sizeof(double)));
  sdf_incubation = (double *)malloc((size_t) (l * sizeof(double)));

  // probability density function of incubation period,
  // pdf_incubation[i] = Pr(incubation duration = i + min_incubation)
  for(t=cfg_pars.min_incubation; t<=cfg_pars.max_incubation; t++)
  {
     r = t - cfg_pars.min_incubation;
     pdf_incubation[r] = cfg_pars.prob_incubation[r];
  }
  // Survival function of incubation period,
  // sdf_incubation[i] = Pr(incubation duration > i + min_incubation)
  for(t=cfg_pars.min_incubation; t<=cfg_pars.max_incubation; t++)
  {
     r = t - cfg_pars.min_incubation;
     if(r == 0)  sdf_incubation[r] = 1.0 - pdf_incubation[r];
     else sdf_incubation[r] = sdf_incubation[r-1] - pdf_incubation[r];
  }

  people = NULL;
  community = NULL;
  value = NULL;

  est = der = value = NULL;

  initialize_matrix(&var);
  initialize_matrix(&var_logit);
  inflate_matrix(&var, n_par, n_par, 0.0);
  inflate_matrix(&var_logit, n_par, n_par, 0.0);
  /*
  printf("n_p_mode=%d\n", cfg_pars.n_p_mode);
  printf("n_q_mode=%d\n", cfg_pars.n_q_mode);
  printf("n_par=%d\n", cfg_pars.n_par);
  printf("n_par_effective=%d\n", cfg_pars.n_par_equiclass);
  printf("asym effect=%f\n", cfg_pars.asym_effect);
  printf("simulation=%d\n", cfg_pars.simulation);
  printf("n_simulation=%d\n", cfg_pars.n_simulation);
  for(j=0; j<cfg_pars.n_par_equiclass; j++)
     printf("%f  ", cfg_pars.sim_par_effective[j]);
  printf("\n");
  printf("EM=%d\n", cfg_pars.EM);
  printf("n_base_sampling=%d\n", cfg_pars.n_base_sampling);
  printf("n_burnin_sampling=%d\n", cfg_pars.n_burnin_sampling);
  printf("n_burnin_iter=%d\n", cfg_pars.n_burnin_iter);
  
  printf("optimization_choice=%d\n", cfg_pars.optimization_choice);
  printf("n_ini=%d\n", cfg_pars.n_ini);
  for(i=0; i<cfg_pars.n_ini; i++)
  {
     for(j=0; j<cfg_pars.n_par_equiclass; j++)
        printf("%f  ", cfg_pars.ini_par_effective[i][j]);
     printf("\n");
  }
  printf("search_bound_provided=%d\n", cfg_pars.search_bound_provided);
  for(i=0; i<cfg_pars.n_par_equiclass; i++)
  {
     printf("%f  %f\n ", cfg_pars.lower_search_bound[i], cfg_pars.upper_search_bound[i]);
  }
  printf("%d  %d\n ", cfg_pars.effective_lower_infectious[0], cfg_pars.effective_upper_infectious[0]);
  exit(0);
  */
  if(n_par > 0)
  {
     est = (double *) malloc((size_t) n_par * sizeof(double));
     der = (double *) malloc((size_t) n_par * sizeof(double));
  }

  if(n_time_ind_covariate > 0 || n_time_dep_covariate > 0)
  {
     k = max(n_time_ind_covariate, n_time_dep_covariate);
     value = (double *) malloc((size_t) k * sizeof(double));
  }

  people = NULL;  community = NULL;
  /************************************************************************************************************/
  /************************************************************************************************************/
  /*** construct population structure ***/
  /************************************************************************************************************/
  /************************************************************************************************************/

  /* This opens the population input file */
  if((file = fopen("pop.dat", "r")) == NULL)
  {
     printf("No valid population file\n");
     goto end;
  }   
  /* determine population file size */
  p_size = 0;
  p_size = get_size(file);
  if(p_size <= 0)
  {
     printf("The number of population should be larger than 0\n");
     fclose(file);
     goto end;
  }   
  else
  {
     rewind(file); 

     /* Allocate space for the people array. */
     people = (PEOPLE *)malloc((size_t) p_size * sizeof(PEOPLE));
     for(i=0,person=people; i < p_size; i++,person++)
     {
        fscanf(file, "%d", &person->id); 
        fscanf(file, "%d", &person->community); 
        fscanf(file, "%d", &person->pre_immune); 
        fscanf(file, "%d", &person->infection); 
        fscanf(file, "%d", &person->symptom); 
        fscanf(file, "%d", &person->day_ill); 
        fscanf(file, "%d", &person->idx); 
        fscanf(file, "%d", &person->u_mode); 
        fscanf(file, "%d", &person->q_mode); 
        fscanf(file, "%d", &person->ignore); 
        //printf("%d  %d  %d\n", person->id, person->community, person->day_ill); 
        person->contact_history = NULL;
        person->risk_history = NULL;
        person->time_ind_covariate = NULL;
        person->time_dep_covariate = NULL;
        person->imm_covariate = NULL;
        person->pat_covariate = NULL;
        person->possible_status = NULL;
        people[i].day_infection_lower = people[i].day_infection_upper = 
        people[i].day_infective_lower = people[i].day_infective_upper = MISSING;
        if(person->infection == 0)  person->day_ill = MISSING; 
        person->size_possible_status = 0;
        person->final_risk_day = MISSING;
     }
     fclose(file); 
  }
  if(cfg_pars.silent_run == 0)  printf("p_size=%d\n", p_size); 
  if((file = fopen("community.dat", "r")) == NULL)
  {
     printf("No valid community file\n");
     goto end;
  }   
  n_community = 0;
  n_community = get_size(file);
  if(n_community <= 0)
  {
     printf("The number of community should be larger than 0\n");
     fclose(file);
     goto end;
  }   
  else
  {
     rewind(file); 

     max_epi_duration = 0;
     community = (COMMUNITY *)malloc((size_t) n_community * sizeof(COMMUNITY));
     for(h=0; h < n_community; h++)
     {
        community[h].idx_day_ill = MISSING;
        community[h].day_epi_start = MISSING;
        community[h].day_epi_stop = MISSING;
        community[h].epi_duration = 0;
        community[h].member = NULL;
        community[h].size = 0;
        community[h].risk_class = NULL;
        community[h].member_impute = NULL;
        community[h].size_impute = 0;
        community[h].size_possible_status = 1;
        community[h].sample_status = NULL;
        community[h].sample_status_rear = NULL;
        community[h].counter = 0;

        fscanf(file, "%d", &community[h].id);  
        fscanf(file, "%d", &community[h].day_epi_start);  //start day of epidemic 
        fscanf(file, "%d", &community[h].day_epi_stop); //stop day of epidemic

        community[h].epi_duration = community[h].day_epi_stop - community[h].day_epi_start + 1;

        max_epi_duration = max(max_epi_duration, community[h].epi_duration);
        //0 < day_epi_start <= day_epi_stop must hold 
        if(community[h].day_epi_stop < community[h].day_epi_start)
        {
           printf("Community %d does not have correctly ordered key days: ",h);
           printf("epidemic start=%d  epidemic stop=%d\n",
                   community[h].day_epi_start, community[h].day_epi_stop);
           fclose(file);
           goto end;
        }
     }
     fclose(file); 
  }
  if(cfg_pars.silent_run == 0)  printf("n_community=%d\n", n_community); 

  for(i=0; i<p_size; i++)
  if(people[i].ignore == 0)
  {
     h = people[i].community;
     community[h].size++;
  }

  for(h=0; h < n_community; h++)
  {
      if(community[h].size == 0) 
         printf("community %d has no individual!\n", community[h].id); 
      else
         community[h].member = (int *) malloc((size_t) community[h].size * sizeof(int));
  }       

  //add non-ignorable people to the communities they belong to
  for(i=0; i<p_size; i++)
  if(people[i].ignore == 0)
  {
     h = people[i].community;
     community[h].member[community[h].counter] = i;
     community[h].counter++;
  }
  /*
  for(i=0; i<5; i++)   
  {
     printf("%d: %d  %d  %d  %d  %d  %d\n", i, people[i].community, people[i].pre_immune, people[i].infection, people[i].u_mode, people[i].q_mode, people[i].ignore);
  }

  for(h=0; h<5; h++)   
  {
     printf("%d: ", h);
     for(i=0; i<community[h].size; i++)
        printf("%d  ", community[h].member[i]);
     printf("\n");
  }
  exit(0);
  */   
  /*********************************************************
    record covariates for each subject, including both
    time-dependent and time-independent covariates.
   *********************************************************/
  for(i=0; i<p_size; i++)
  if(people[i].ignore == 0)
  {
     h = people[i].community;
     if(n_time_ind_covariate > 0)
        people[i].time_ind_covariate = (double *) malloc((size_t) (n_time_ind_covariate * sizeof(double)));
     if(n_time_dep_covariate > 0)
	make_2d_array_double(&people[i].time_dep_covariate, community[h].epi_duration, n_time_dep_covariate, 0.0);
  }   

  if(n_time_ind_covariate > 0)
  {
     if((file = fopen("time_ind_covariate.dat", "r")) == NULL)
     {
        printf("No valid time-independent covariates file\n");
        goto end;
     }   
     n_record = get_size(file);
     rewind(file); 

     for(m=0; m<n_record; m++)
     {
        fscanf(file, "%d", &i); 
        for(j=0; j<n_time_ind_covariate; j++)
           fscanf(file, "%lf", &value[j]); 
        if(people[i].ignore == 0)
        {
           for(j=0; j<n_time_ind_covariate; j++)
              people[i].time_ind_covariate[j] = value[j];
        }
     }
     fclose(file);
  }
  
  if(n_time_dep_covariate > 0)
  {
     if((file = fopen("time_dep_covariate.dat", "r")) == NULL)
     {
        printf("No valid time-dependent covariates file\n");
        goto end;
     }   
     n_record = get_size(file);
     rewind(file); 
  
     for(m=0; m<n_record; m++)
     {
        /*time-dependent covariates are stored as subject, covariate index, day, value per line*/
        fscanf(file, "%d  %d", &i, &t); 
        for(k=0; k<n_time_dep_covariate; k++)
           fscanf(file, "%lf", &value[k]); 
        if(people[i].ignore == 0)
        {
           if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
           {
              r = t - community[h].day_epi_start;
              for(k=0; k<n_time_dep_covariate; k++)
                 people[i].time_dep_covariate[r][k] = value[k];
           }
        } 
     }   
     fclose(file);
  }
  /*
  printf("n_record=%d\n", n_record);
  printf("%d %d\n", n_time_ind_covariate, n_time_dep_covariate);
  printf("%f %f\n", people[p_size-1].time_ind_covariate[0], people[p_size-1].time_dep_covariate[0][0]);
  exit(0);
  */ 

  for(i=0; i<p_size; i++)
  if(people[i].ignore == 0)
  {
     h = people[i].community;
     people[i].contact_history = (CONTACT_HISTORY *)malloc((size_t) (community[h].epi_duration * sizeof(CONTACT_HISTORY)));
     people[i].risk_history = (RISK_HISTORY *)malloc((size_t) (community[h].epi_duration * sizeof(RISK_HISTORY)));
     for(t=community[h].day_epi_start; t<=community[h].day_epi_stop; t++)
     {
        r = t - community[h].day_epi_start;
        people[i].contact_history[r].c2p_contact = NULL;
        people[i].contact_history[r].c2p_contact_rear = NULL;
        people[i].contact_history[r].p2p_contact = NULL;
        people[i].contact_history[r].p2p_contact_rear = NULL;
        people[i].risk_history[r].c2p_risk = NULL;
        people[i].risk_history[r].c2p_risk_rear = NULL;
        people[i].risk_history[r].p2p_risk = NULL;
        people[i].risk_history[r].p2p_risk_rear = NULL;
     }
     
     if(n_pat_covariate > 0)  
     {
        make_2d_array_double(&people[i].pat_covariate, n_pat_covariate, community[h].epi_duration, 0.0);
        organize_pat_covariate(people + i);
     }

     if(n_imm_covariate > 0)  
     {
        make_1d_array_double(&people[i].imm_covariate, n_imm_covariate, 0.0);
        organize_imm_covariate(people + i);
     }
  }

  // ad-hoc creation of c2p-contact file
  
  if(cfg_pars.generate_c2p_contact == 1 && cfg_pars.common_contact_history_within_community == 1 && n_b_mode == 1)
  {
     for(h=0; h<n_community; h++)
     if(community[h].size > 0)
     {
        for(t = community[h].day_epi_start; t <= community[h].day_epi_stop; t++)
        {
           for(l=0; l<community[h].size; l++)
           {
              i = community[h].member[l];
              if(people[i].ignore != 1)
              {
                 add_c2p_contact_history(t, people + i, 0, 0.0);
              }
           }
        }
     }
  }
  // ad-hoc creation of p2p-contact file
  
  if(cfg_pars.generate_p2p_contact == 1 && cfg_pars.common_contact_history_within_community == 1 && n_p_mode == 1)
  {
     for(h=0; h<n_community; h++)
     if(community[h].size > 0)
     {
        for(t = community[h].day_epi_start; t <= community[h].day_epi_stop; t++)
        {
           for(k=0; k<community[h].size; k++)
           {
              i = community[h].member[k];
              for(l=k+1; l<community[h].size; l++)
              {
                 j = community[h].member[l];
                 if(people[i].ignore != 1 && people[j].ignore != 1)
                 {
                    add_p2p_contact_history(t, people + j, people + i, 0, 0.0);
                    add_p2p_contact_history(t, people + i, people + j, 0, 0.0);
                 }
              }
           }
        } 
     }     
  }
  

  /************************************************************* 
  If susceptible subject i is exposed to common source of some type, 
  record this exposure in the structure people[i].contact_history[r].c2p_contact. 
  **************************************************************/
  /* This opens the special exposure input file */
  if(n_b_mode > 0 && cfg_pars.generate_c2p_contact == 0)
  {
     if(cfg_pars.silent_run == 0)  printf("Reading c2p contact information\n");
     if((file = fopen("c2p_contact.dat", "r")) == NULL)
     {
        printf("No valid c2p_contact information file\n");
        goto end;
     }   
  
     n_record = get_size(file);
     rewind(file);
     // when people in the same community share the same contact history,
     // the contact input files can be based on communities, which will simplify
     // the input files greatly, especially for large population. 
     if(cfg_pars.common_c2p_contact_history_across_community == 1)
     {
        for(m=0; m<n_record; m++)
        {
           fscanf(file, "%d  %d  %d  %lf  %d\n", &start_day, &stop_day, &contact_mode, &offset, &ignore);
           if(cfg_pars.c2p_offset == 0)  offset = 0.0;
           if(ignore != 1)
           {
              for(t = start_day; t <= stop_day; t++)
              {
                 for(h=0; h<n_community; h++)
                 {
                    if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
                    for(l=0; l<community[h].size; l++)
                    {
                       i = community[h].member[l];
                       if(people[i].ignore != 1)
                       {
                          add_c2p_contact_history(t, people + i, contact_mode, offset);
                       }
                    }
                 }
              }
           }
        } 
     }
     else if(cfg_pars.common_contact_history_within_community == 1)
     {
        //when people in the community share the same contact history,
        //we only need to know which contact for which days. 
        for(m=0; m<n_record; m++)
        {
           fscanf(file, "%d  %d  %d  %d  %lf  %d\n", &h, &start_day, &stop_day, &contact_mode, &offset, &ignore);
           if(cfg_pars.c2p_offset == 0)  offset = 0.0;
           if(ignore != 1 && community[h].size > 0)
           {
              for(t = start_day; t <= stop_day; t++)
              {
                 if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
                 {
                    for(l=0; l<community[h].size; l++)
                    {
                       i = community[h].member[l];
                       if(people[i].ignore != 1)
                       {
                          add_c2p_contact_history(t, people + i, contact_mode, offset);
                       }
                    }
                 }
              }
           }
        } 
     }
     else
     {
        //Individualized contact history
        for(m=0; m<n_record; m++)
        {
           fscanf(file, "%d  %d  %d  %d  %lf  %d\n", &i, &start_day, &stop_day, &contact_mode, &offset, &ignore); 
           if(cfg_pars.c2p_offset == 0)  offset = 0.0;
           if(ignore != 1 && people[i].ignore != 1)
           {
              h = people[i].community;
              for(t = start_day; t <= stop_day; t++)
              {
                 if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
                 {
                    add_c2p_contact_history(t, people + i, contact_mode, offset);
                 }
              }
           }
        }
     }   
     fclose(file);
  }

  /*
  for(i=0; i<p_size; i++)
  {
     h = people[i].community;
     for(t=community[h].day_epi_start; t<=community[h].day_epi_stop; t++)
     {
        r = t - community[h].day_epi_start;
        if(people[i].id == 0 | people[i].id == p_size-1)
        {
           ptr_contact = people[i].contact_history[r].c2p_contact;
           while(ptr_contact != NULL)
           {
              printf("id=%d  day=%d  contact_mode:%d  offset:%f  covariate:",
                     people[i].id, t, ptr_contact->contact_mode,
                     ptr_contact->offset);
              for(l=0; l<n_c2p_covariate; l++)
                 printf("%f  ", ptr_contact->covariate[l]);
              printf("\n");
              ptr_contact = ptr_contact->next;
           }
        }
     }
  }
  exit(0);
  */
  /************************************************************* 
  If subject i made contact with subject j when i is susceptible and
  j is infectious, record this contact in the structure
  people[i].contact_history[r].p2p_contact. 
  **************************************************************/

  
  if(n_p_mode > 0 && cfg_pars.generate_p2p_contact == 0)
  {
     if(cfg_pars.silent_run == 0)  printf("Reading p2p contact information\n");
     if((file = fopen("p2p_contact.dat", "r")) == NULL)
     {
        printf("No valid p2p_contact information file\n");
        goto end;
     }   
  
     n_record = get_size(file);
     rewind(file); 
  
     if(cfg_pars.common_contact_history_within_community == 1)
     {
        //when people in the community share the same contact history,
        //we only need to know which contact for which days. 
        for(m=0; m<n_record; m++)
        {
           fscanf(file, "%d  %d  %d  %d  %lf  %d\n", &h, &start_day, &stop_day, &contact_mode, &offset, &ignore);
           if(cfg_pars.p2p_offset == 0)  offset = 0.0;
           if(ignore != 1 && community[h].size > 0)
           {
              for(t = start_day; t <= stop_day; t++)
              {
                 if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
                 {
                    for(k=0; k<community[h].size; k++)
                    {
                       i = community[h].member[k];
                       for(l=k+1; l<community[h].size; l++)
                       {
                          j = community[h].member[l];
                          if(people[i].ignore != 1 && people[j].ignore != 1)
                          {
                             add_p2p_contact_history(t, people + j, people + i, contact_mode, offset);
                             add_p2p_contact_history(t, people + i, people + j, contact_mode, offset);
                          }
                       }
                    }
                 } 
              }     
           }  
        }
     }
     else
     {
        //Individualized contact history
        for(m=0; m<n_record; m++)
        {
           fscanf(file, "%d  %d  %d  %d  %d  %lf  %d\n", &start_day, &stop_day, &i, &j, &contact_mode, &offset, &ignore);
           if(cfg_pars.p2p_offset == 0)  offset = 0.0;
           if(ignore != 1 && people[i].ignore == 0 && people[j].ignore == 0 
              && people[i].community == people[j].community && people[i].id != people[j].id)
           {
              h = people[i].community;
              for(t = start_day; t <= stop_day; t++)
              {
                 if(t >= community[h].day_epi_start && t <= community[h].day_epi_stop)
                 {
                    add_p2p_contact_history(t, people + j, people + i, contact_mode, offset);
                    add_p2p_contact_history(t, people + i, people + j, contact_mode, offset);
                 } 
              }     
           }  
        }
     }
     fclose(file); 
  }
  
  /*
  for(i=0; i<p_size; i++)
  {
     h = people[i].community;
     for(t=community[h].day_epi_start; t<=community[h].day_epi_stop; t++)
     {
        r = t - community[h].day_epi_start;
        if(people[i].id == 249)
        {
           ptr_contact = people[i].contact_history[r].p2p_contact;
           while(ptr_contact != NULL)
           {
              printf("i=%d  day=%d  j=%d  contact_mode:%d  offset:%f  covariate:",
                     people[i].id, t, ptr_contact->contact_id, ptr_contact->contact_mode,
                     ptr_contact->offset);
              for(l=0; l<n_p2p_covariate; l++)
                 printf("%f  ", ptr_contact->covariate[l]);
              printf("\n");
              ptr_contact = ptr_contact->next;
           }
        }
     }
  }
  exit(0);
  */
  create_arrays();
  if(cfg_pars.simulation == 0)
  {
     system("rm -f estimates.txt");
     system("rm -f errors.txt");
  }

  n_iter = (cfg_pars.simulation == 1)? cfg_pars.n_simulation : 1;
  for(ITER=0; ITER<n_iter; ITER++)
  {
     if(cfg_pars.simulation == 1 && cfg_pars.silent_run == 0)  printf(" SIMULATION: %d \n\n", ITER);
     if(cfg_pars.simulation == 1)  simulate();
     // From now on an epidemic has been generated, either from user input files or the simulator

     //we search the earliest user specified index case in the community, and treat that as
     //the base day for left-truncation adjustment. 
     //Attention: if a case has symptom onset way earlier than other index cases, it would be better
     //not to denote it as an index case in pop.dat, but let TranStat to set it as an index case,
     //so that TranStat does not set that case's symptom onset day as community[h].idx_day_ill.
     for(h=0; h < n_community; h++)
     {
        for(j=0; j<community[h].size; j++)
        {
           i = community[h].member[j];
           if(people[i].idx == 1 && people[i].infection == 1)
           {
              if(community[h].idx_day_ill == MISSING)
                 community[h].idx_day_ill = people[i].day_ill;
              else
                 community[h].idx_day_ill = min(community[h].idx_day_ill, people[i].day_ill);
           }
        }
     }
     
     //People with earlier symptom onset days than community[h].idx_day_ill
     //but not user-specified as index cases will be regarded as index cases
     //automatically, but they do not affect the base day for left-truncation adjustment,
     //because they are not the reason the community is selected. We mark them
     // as index cases simply to exclude them from the likelihood. Like user-specified
     //index cases, they do contribute to the exposure history of secondary cases though.

     for(h=0; h < n_community; h++)
     {
        for(j=0; j<community[h].size; j++)
        {
           i = community[h].member[j];
           if(people[i].idx == 0 && people[i].infection == 1 && people[i].symptom == 1 && community[h].idx_day_ill != MISSING 
              && people[i].day_ill <= community[h].idx_day_ill)
              people[i].idx = 1;
        }
     }   
     /*
     printf("p_size=%d  n_community=%d\n", p_size, n_community);
     for(h=0; h<n_community; h++)
     {
        printf("%d: size=%d  day_epi_start=%d  day_epi_stop=%d  idx_day_ill=%d\n", 
                h, community[h].size, community[h].day_epi_start, community[h].day_epi_stop,
                community[h].idx_day_ill);
     }
     goto end;
     */
     // For case-ascertained study, it's important to exclude
     // communities free of the disease to avoid biased estimates.
     if(cfg_pars.adjust_for_left_truncation == 1)
     {
        for(h=0; h < n_community; h++)
        {
           if(community[h].idx_day_ill == MISSING)
           {
              for(j=0; j<community[h].size; j++)
              {
                 i = community[h].member[j];
                 people[i].ignore = 1;
              }
              if(community[h].member != NULL) 
              {
                 free(community[h].member);
                 community[h].member = NULL;
              }
              community[h].size = 0;
           }
        }
     }

     // Create risk classes for each community.
     // For cases-ascertained design, index cases are excluded from any risk class.
     // People in the same class share contact history and covariates affecting risk.
     // Such classes expedite the analysis quite a lot.
     create_risk_class();
     /*
     file = fopen("check_risk_class.dat", "w");
     for(h=0; h<n_community; h++)
     {
        fprintf(file, "%d:\n", h);
        ptr_class = community[h].risk_class;
        while(ptr_class != NULL)
        {
           ptr_integer = ptr_class->member;
           while(ptr_integer != NULL)
           {
              fprintf(file, "%d  ", ptr_integer->id);
              ptr_integer = ptr_integer->next;
           }
           fprintf(file, "\n");
           ptr_class = ptr_class->next;
        }
     }   
     fclose(file);
     */

     n_need_OEM = n_need_MCEM = 0;
     if(cfg_pars.EM == 1)
     {
        if(cfg_pars.simulation == 0)
        {
           if((file = fopen("impute.dat", "r")) == NULL)
           {
              printf("No valid imputation information file\n");
              goto end;
           }   
        
           n_record = get_size(file);
           rewind(file); 
           for(m=0; m<n_record; m++)
           {
              fscanf(file, "%d", &i); 
              fscanf(file, "%d", &possible_pre_immune); 
              fscanf(file, "%d", &possible_escape); 
              fscanf(file, "%d  %d  %d", &possible_sym, &possible_sym_start, &possible_sym_stop); 
              fscanf(file, "%d  %d  %d", &possible_asym, &possible_asym_start, &possible_asym_stop); 
              if(people[i].ignore != 1)
              {
                 people[i].size_possible_status = 0;
                 if(possible_pre_immune == 1)  people[i].size_possible_status++;
                 if(possible_escape == 1)  people[i].size_possible_status++;
                 if(possible_sym == 1)  people[i].size_possible_status += (possible_sym_stop - possible_sym_start + 1);
                 if(possible_asym == 1)  people[i].size_possible_status += (possible_asym_stop - possible_asym_start + 1);
                 if(people[i].size_possible_status > 0)
                 {
                    make_2d_array_int(&people[i].possible_status, 2, people[i].size_possible_status, 0);
                    k = 0;
                    if(possible_pre_immune == 1)
                    {
                       people[i].possible_status[0][k] = -INFINITY;
                       k++;
                    }
                    if(possible_escape == 1)
                    {
                       people[i].possible_status[0][k] = INFINITY;
                       k++;
                    }
                    if(possible_sym == 1)
                    {
                       for(j=possible_sym_start; j<=possible_sym_stop; j++)
                       {
                          people[i].possible_status[0][k] = j;
                          people[i].possible_status[1][k] = 1;
                          k++;
                       }
                    }
                    if(possible_asym == 1)
                    {
                       for(j=possible_asym_start; j<=possible_asym_stop; j++)
                       {
                          people[i].possible_status[0][k] = j;
                          people[i].possible_status[1][k] = 0;
                          k++;
                       }
                    }
                 }
              }
           }
           fclose(file);
        }
        else
        {
           for(i=0; i<p_size; i++)
           {
              if(people[i].ignore != 1 && people[i].infection == 1 && people[i].symptom == 0)
              {
                 h = people[i].community;
                 //shortest incubation period is 1 day, so earliest possible infection day is 2.
                 possible_asym_start = community[h].day_epi_start + cfg_pars.min_incubation;
                 possible_asym_stop = community[h].day_epi_stop;
                 people[i].size_possible_status = (possible_asym_stop - possible_asym_start + 1);
                 make_2d_array_int(&people[i].possible_status, 2, people[i].size_possible_status, 0);
                 k = 0;
                 for(j=possible_asym_start; j<=possible_asym_stop; j++)
                 {
                    people[i].possible_status[0][k] = j;
                    people[i].possible_status[1][k] = 0;
                    k++;
                 }
              }
              // for 50% of preimmune or escaped people, we assume pre-immunity and escape are not distinguishable
              
              if(people[i].ignore != 1 && people[i].infection == 0 && people[i].symptom == 0 && cfg_pars.n_q_mode > 0)
              {
                 if(runiform(&seed) < cfg_pars.prop_mix_imm_esc)
                 {
                    people[i].size_possible_status = 2;
                    make_2d_array_int(&people[i].possible_status, 2, people[i].size_possible_status, 0);
                    people[i].possible_status[0][0] = -INFINITY;
                    people[i].possible_status[0][1] = INFINITY;
                 }
              }
           }
        } //if(cfg_pars.simulation == 0)

        //any subject need imputation in this community
        for(h=0; h < n_community; h++)
        {
           for(j=0; j<community[h].size; j++)
           {
              i = community[h].member[j];
              if(people[i].size_possible_status > 0)  
              {
                 community[h].size_impute ++;
                 community[h].size_possible_status *= people[i].size_possible_status;
              } 
           }
           if(community[h].size_impute > 0)
           {
              community[h].member_impute = (int *) malloc((size_t) community[h].size_impute * sizeof(int));
              k = 0;
              for(j=0; j<community[h].size; j++)
              {
                 i = community[h].member[j];
                 if(people[i].size_possible_status > 0)  
                 {
                    community[h].member_impute[k++] = i;
                 }
              }
           }
           // if size_impute is <= 1 in all communities, we can do straightforward EM rather than MCEM.
           if(community[h].size_possible_status > 1 && 
              community[h].size_possible_status < cfg_pars.min_size_MCEM)  n_need_OEM++;
           if(community[h].size_possible_status >= cfg_pars.min_size_MCEM)  n_need_MCEM++;
        }
        if(cfg_pars.silent_run == 0)  printf("number of communities need OEM or MCEM=%d %d\n", n_need_OEM, n_need_MCEM);
     } //if(cfg_pars.EM == 1)
     /*
     k = 0;
     n_sym = n_asym = n_esc = n_imm = 0;
     for(i=0; i < p_size; i++)
     {
        if(people[i].pre_immune == 1)  n_imm++;
        if(people[i].infection == 0)  n_esc++;
        else
        {
           if(people[i].symptom == 1)  n_sym++;
           else  n_asym++;
        }
        if(people[i].size_possible_status > 0)  k++;
     }
     
     for(i=0; i<p_size; i++)
     if(people[i].size_possible_status > 0)
     {
        printf("%d: size=%d  infection=%d  symptom=%d  day_ill=%d\n", 
                i, people[i].size_possible_status, people[i].infection, people[i].symptom, people[i].day_ill);
        for(j=0; j<people[i].size_possible_status; j++)
           printf("%d  ", people[i].possible_status[0][j]);
        printf("\n");
        for(j=0; j<people[i].size_possible_status; j++)
           printf("%d  ", people[i].possible_status[1][j]);
        printf("\n\n");
        break;
     }
     exit(0);
     */

     //day_epi_start is considered the first day for any possible exposure.
     //Cases without any possible infection day >= day_epi_start, 
     //they will contribute to exposure of other members, but their own disease outcome
     //should not contribute to the likelihood for tranmission or pathogenicity.
     //day_epi_stop is considered the last observation day. 
     //for case with ILI onset after day_epi_stop, set the cases as none-cases,
     //and user can specify adjustment for right-censoring to counter-balance
     //possible bias caused by this misclassification. 
     for(i=0; i<p_size; i++)
     if(people[i].ignore == 0 && people[i].infection == 1 && people[i].size_possible_status == 0)
     {
        h = people[i].community;
        if(h < 0 || h >= n_community)
        {
           printf("person %d has an invalid community id %d\n", people[i].id, h);
           goto end;
        }
        // Do not set ignore=1 as this person may still contribute exposre information for susceptibles
        //if((people[i].day_ill - cfg_pars.min_incubation) < community[h].day_epi_start)
        //   people[i].ignore = 1;
        if(people[i].day_ill > community[h].day_epi_stop)
        {
           people[i].infection = 0;
           people[i].symptom = 0;
           people[i].day_ill = MISSING;
        }
        people[i].day_infection_lower = max(people[i].day_ill - cfg_pars.max_incubation, community[h].day_epi_start);
        people[i].day_infection_upper  = people[i].day_ill - cfg_pars.min_incubation;

        if(people[i].day_ill + cfg_pars.lower_infectious <= community[h].day_epi_stop)
        { 
           people[i].day_infective_lower = people[i].day_ill + cfg_pars.lower_infectious;
           people[i].day_infective_upper  = min(people[i].day_ill + cfg_pars.upper_infectious, community[h].day_epi_stop);
        }
     }
     
     //Create risk history. First we need to initialize infection status for people with uncertain status.
     //final_risk_day is used to track the last exposure day for people for whom imputation is not neccesary.
     //When imputing infection status of a subject, we need to update the risk history of his contacts;
     //but if we are sure a contact is not susceptible anymore, such updating can be skipped, which
     // will save some computation. 
     for(h=0; h<n_community; h++)
     if(community[h].size > 0)
     {
        for(m=0; m<community[h].size; m++)
        {
           person = people + community[h].member[m];
           if(person->size_possible_status > 0)
           {
              person->current_status = floor(person->size_possible_status * runiform(&seed));
              l = person->current_status;
              if(person->possible_status[0][l] == -INFINITY) //pre-immune
              {
                 person->pre_immune = 1;
                 person->infection = 0;
                 person->symptom = 0;
                 person->day_ill = MISSING;
              }
              else if(person->possible_status[0][l] == INFINITY) //escape
              {
                 person->pre_immune = 0;
                 person->infection = 0;
                 person->symptom = 0;
                 person->day_ill = MISSING;
              }
              else // symptomatic or asymptomatic infection
              {
                 person->pre_immune = 0;
                 person->infection = 1;
                 person->symptom = person->possible_status[1][l];
                 person->day_ill = person->possible_status[0][l];
                 if(person->day_ill + cfg_pars.lower_infectious <= community[h].day_epi_stop)
                 {
                    person->day_infective_lower = person->day_ill + cfg_pars.lower_infectious;
                    person->day_infective_upper = min(person->day_ill + cfg_pars.upper_infectious, community[h].day_epi_stop);
                 }
                 person->day_infection_lower = max(person->day_ill - cfg_pars.max_incubation, community[h].day_epi_start);
                 person->day_infection_upper  = person->day_ill - cfg_pars.min_incubation;
              }
           }
           else
           {
              if(person->pre_immune == 1)  person->final_risk_day = -INFINITY;
              else
              {
                 if(person->infection == 1)  person->final_risk_day = person->day_ill - cfg_pars.min_incubation;
                 else  person->final_risk_day = community[h].day_epi_stop;
              }
           }
        }
     }

     // output the population
     /*
     file = fopen("check_pop.dat", "w");
     for(i=0; i<p_size; i++)
     {
        fprintf(file, "%d  %d  preimmune=%d  infection=%d  symptom=%d  day_ill=%d  idx=%d  ignore=%d\n", 
             i, people[i].community,people[i].pre_immune,  people[i].infection, 
             people[i].symptom, people[i].day_ill, people[i].idx, people[i].ignore);
     }
     fclose(file);
     */
     /*
     file = fopen("pop_mock.dat", "w");
     for(i=0; i<p_size; i++)
     {
        fprintf(file, "%d  %d  %d  %d  %d  %d  %d\n", 
             i, people[i].community, people[i].infection, people[i].day_ill, people[i].idx, 0, people[i].ignore);
     }
     fclose(file);
     */
     // now we are ready to build the risk history based on the contact history.
     for(i=0; i<p_size; i++)
     {
        person = people + i;
        if(person->ignore == 0)
        {
           h = person->community;
           //build c2p risk history
           for(t=community[h].day_epi_start; t<=community[h].day_epi_stop; t++)
           if(!(person->final_risk_day != MISSING && t > person->final_risk_day))
           {
              r = t - community[h].day_epi_start;

              ptr_contact = person->contact_history[r].c2p_contact;
              while(ptr_contact != NULL)
              {
                 add_c2p_risk_history(t, person, ptr_contact);
                 ptr_contact = ptr_contact->next;
              }
           }
           //build p2p risk history
           if(person->infection == 1 && 
              person->day_infective_lower != MISSING && person->day_infective_upper != MISSING)
           {
              for(t=person->day_infective_lower; t<=person->day_infective_upper; t++)
              {
                 r = t - community[h].day_epi_start;
                 infective_prob = cfg_pars.prob_infectious[t - person->day_infective_lower];
                 //printf("infective=%d  t=%d  infective_prob=%f\n", person->id, t, infective_prob);
                 ptr_contact = person->contact_history[r].p2p_contact;
                 while(ptr_contact != NULL)
                 {
                    member = people + ptr_contact->contact_id;
                    //if(h == 0)  printf("member=%d  final risk day=%d\n", member->id, member->final_risk_day);
                    if(!(member->final_risk_day != MISSING && t > member->final_risk_day))
                       add_p2p_risk_history(t, member, ptr_contact, infective_prob, person->symptom);
                    ptr_contact = ptr_contact->next;
                 }
              }
           }
        }
     } 
     // print out risk history for an arbitrary person to check if we get this part correctly
     /*
     if(ITER >= 0)
     {
        for(i=0; i<p_size; i++)
        if(i == 0)
        {
           h = people[i].community;
           printf("c2p risk history:\n");
           show_c2p_risk(people+i, community[h].day_epi_start, community[h].day_epi_stop);
           printf("p2p risk history:\n");
           show_p2p_risk(people+i, community[h].day_epi_start, community[h].day_epi_stop);
        }
        exit(0); 
     } 
     */
     if(cfg_pars.simulation == 0)
     {
        if(cfg_pars.silent_run == 0)  printf("Checking admissibility now\n");
        if(admissibility() == 1)  goto end;
     }
     estimation_error_type = test_error_type = -1;
     
     /*********************************************************************************
     Estimation starts here.
     estimation_error_type takes 3 values:
     0: no error
     1: The maximum number of function evaluation in Nelder_mead searching is reached
     2: error in variance calculating, which means the final estimates may not be valid
     ***********************************************************************************/
     estimation_error_type = estimation(est, &log_likelihood, &var, &var_logit);
     
     if(cfg_pars.simulation == 0)
     {      
        if((file = fopen("errors.txt", "a")) == NULL)
           file = fopen("errors.txt", "w");
        fprintf(file, "\n#1.3 Estimation error: \n");
        fprintf(file, "%d\n", estimation_error_type);
        fclose(file);
     }
     

     if(estimation_error_type == 0)
     {
        for(k=0; k<n_b_mode; k++)  
        {  
           b[k] = est[k];
           lb[k] = logit(b[k]);
           se_lb[k] = sqrt(var_logit.data[k][k]);   
           se_b[k] = sqrt(var.data[k][k]);   
        }   
        for(k=0; k<n_p_mode; k++)  
        {
           m = n_b_mode + k;           
           p[k] = est[m];
           lp[k] = logit(p[k]);
           se_lp[k] = sqrt(var_logit.data[m][m]);   
           se_p[k] = sqrt(var.data[m][m]);   
        }   
        for(k=0; k<n_u_mode; k++)  
        {
           m = n_b_mode + n_p_mode + k;           
           u[k] = est[m];
           lu[k] = logit(u[k]);
           se_lu[k] = sqrt(var_logit.data[m][m]);   
           se_u[k] = sqrt(var.data[m][m]);   
        }   
        for(k=0; k<n_q_mode; k++)  
        {
           m = n_b_mode + n_p_mode + n_u_mode + k;           
           q[k] = est[m];
           lq[k] = logit(q[k]);
           se_lq[k] = sqrt(var_logit.data[m][m]);   
           se_q[k] = sqrt(var.data[m][m]);   
        }   
        for(k=0; k<n_c2p_covariate; k++)  
        {
           m = n_b_mode + n_p_mode + n_u_mode + n_q_mode + k;           
           OR_c2p[k] = est[m];
           coeff_c2p[k] = log(OR_c2p[k]);
           se_coeff_c2p[k] = sqrt(var_logit.data[m][m]);   
           se_OR_c2p[k] = sqrt(var.data[m][m]);   
        }   
        for(k=0; k<n_p2p_covariate; k++)  
        {
           m = n_b_mode + n_p_mode + n_u_mode + n_q_mode + n_c2p_covariate + k;
           OR_p2p[k] = est[m];
           coeff_p2p[k] = log(OR_p2p[k]);
           se_coeff_p2p[k] = sqrt(var_logit.data[m][m]);   
           se_OR_p2p[k] = sqrt(var.data[m][m]);   
        }
        for(k=0; k<n_pat_covariate; k++)  
        {
           m = n_b_mode + n_p_mode + n_u_mode + n_q_mode + n_c2p_covariate + n_p2p_covariate + k;
           OR_pat[k] = est[m];
           coeff_pat[k] = log(OR_pat[k]);
           se_coeff_pat[k] = sqrt(var_logit.data[m][m]);   
           se_OR_pat[k] = sqrt(var.data[m][m]);   
        }
        for(k=0; k<n_imm_covariate; k++)  
        {
           m = n_b_mode + n_p_mode + n_u_mode + n_q_mode + n_c2p_covariate + n_p2p_covariate + n_pat_covariate;
           OR_imm[k] = est[m];
           coeff_imm[k] = log(OR_imm[k]);
           se_coeff_imm[k] = sqrt(var_logit.data[m][m]);   
           se_OR_imm[k] = sqrt(var.data[m][m]);   
        }
        
     
        if(cfg_pars.simulation == 1)
        {
           sprintf(file_name, "sim_%d.dat", serial_number);
           if((file = fopen(file_name, "a")) == NULL)
              file = fopen(file_name, "w");

           for(k=0; k<n_b_mode; k++)
           {
              lower = inv_logit(lb[k] - 1.96 * se_lb[k]);
              upper = inv_logit(lb[k] + 1.96 * se_lb[k]);
              fprintf(file, "%e  %e  %e  %e  ", b[k], se_b[k], lower, upper);
           }   
           
           for(k=0; k<n_p_mode; k++)
           {
              lower = inv_logit(lp[k] - 1.96 * se_lp[k]);
              upper = inv_logit(lp[k] + 1.96 * se_lp[k]);
              fprintf(file, "%e  %e  %e  %e  ", p[k], se_p[k], lower, upper);
           }   
              
           for(k=0; k<n_u_mode; k++)
           {
              lower = inv_logit(lu[k] - 1.96 * se_lu[k]);
              upper = inv_logit(lu[k] + 1.96 * se_lu[k]);
              fprintf(file, "%e  %e  %e  %e  ", u[k], se_u[k], lower, upper);
           }   
              
           for(k=0; k<n_q_mode; k++)
           {
              lower = inv_logit(lq[k] - 1.96 * se_lq[k]);
              upper = inv_logit(lq[k] + 1.96 * se_lq[k]);
              fprintf(file, "%e  %e  %e  %e  ", q[k], se_q[k], lower, upper);
           }   
              
           for(k=0; k<n_c2p_covariate; k++)
           {
              lower = exp(coeff_c2p[k] - 1.96 * se_coeff_c2p[k]);
              upper = exp(coeff_c2p[k] + 1.96 * se_coeff_c2p[k]);
              fprintf(file, "%7.3f  %7.3f  %7.3f,  %7.3f  ", OR_c2p[k], se_OR_c2p[k], lower, upper);
           }   
           
           for(k=0; k<n_p2p_covariate; k++)
           {
              lower = exp(coeff_p2p[k] - 1.96 * se_coeff_p2p[k]);
              upper = exp(coeff_p2p[k] + 1.96 * se_coeff_p2p[k]);
              fprintf(file, "%7.3f  %7.3f  %7.3f,  %7.3f  ", OR_p2p[k], se_OR_p2p[k], lower, upper);
           }   

           for(k=0; k<n_pat_covariate; k++)
           {
              lower = exp(coeff_pat[k] - 1.96 * se_coeff_pat[k]);
              upper = exp(coeff_pat[k] + 1.96 * se_coeff_pat[k]);
              fprintf(file, "%7.3f  %7.3f  %7.3f,  %7.3f", OR_pat[k], se_OR_pat[k], lower, upper);
           }   
           for(k=0; k<n_imm_covariate; k++)
           {
              lower = exp(coeff_imm[k] - 1.96 * se_coeff_imm[k]);
              upper = exp(coeff_imm[k] + 1.96 * se_coeff_imm[k]);
              fprintf(file, "%7.3f  %7.3f  %7.3f,  %7.3f", OR_imm[k], se_OR_imm[k], lower, upper);
           }   
           fprintf(file, "\n");
           fclose(file);
        }
        else
        {
           file = fopen("estimates.txt", "w");
     
           fprintf(file, "\n# Estimates of b\n");
           for(k=0; k<n_b_mode; k++)
           {
              lower = inv_logit(lb[k] - 1.96 * se_lb[k]);
              upper = inv_logit(lb[k] + 1.96 * se_lb[k]);
              fprintf(file, "%e,  %e,  %e,  %e\n", b[k], se_b[k], lower, upper);
           }   
           
           fprintf(file, "\n# Estimates of p\n");
           for(k=0; k<n_p_mode; k++)
           {
              lower = inv_logit(lp[k] - 1.96 * se_lp[k]);
              upper = inv_logit(lp[k] + 1.96 * se_lp[k]);
              fprintf(file, "%e,  %e,  %e,  %e\n", p[k], se_p[k], lower, upper);
           }   
              
           fprintf(file, "\n# Estimates of u\n");
           for(k=0; k<n_u_mode; k++)
           {
              lower = inv_logit(lu[k] - 1.96 * se_lu[k]);
              upper = inv_logit(lu[k] + 1.96 * se_lu[k]);
              fprintf(file, "%e,  %e,  %e,  %e\n", u[k], se_u[k], lower, upper);
           }   
              
           fprintf(file, "\n# Estimates of q\n");
           for(k=0; k<n_q_mode; k++)
           {
              lower = inv_logit(lq[k] - 1.96 * se_lq[k]);
              upper = inv_logit(lq[k] + 1.96 * se_lq[k]);
              fprintf(file, "%e,  %e,  %e,  %e\n", q[k], se_q[k], lower, upper);
           }   
              
           fprintf(file, "\n# Estimates of odds ratios for c2p exposure\n");
           for(k=0; k<n_c2p_covariate; k++)
           {
              lower = exp(coeff_c2p[k] - 1.96 * se_coeff_c2p[k]);
              upper = exp(coeff_c2p[k] + 1.96 * se_coeff_c2p[k]);
              fprintf(file, "%20.8f,  %20.8f,  %20.8f,  %20.8f\n", OR_c2p[k], se_OR_c2p[k], lower, upper);
           }   
           
           fprintf(file, "\n# Estimates of odds ratios for p2p exposure\n");
           for(k=0; k<n_p2p_covariate; k++)
           {
              lower = exp(coeff_p2p[k] - 1.96 * se_coeff_p2p[k]);
              upper = exp(coeff_p2p[k] + 1.96 * se_coeff_p2p[k]);
              fprintf(file, "%20.8f,  %20.8f,  %20.8f,  %20.8f\n", OR_p2p[k], se_OR_p2p[k], lower, upper);
           }   

           fprintf(file, "\n# Estimates of odds ratios for pathogenicity\n");
           for(k=0; k<n_pat_covariate; k++)
           {
              lower = exp(coeff_pat[k] - 1.96 * se_coeff_pat[k]);
              upper = exp(coeff_pat[k] + 1.96 * se_coeff_pat[k]);
              fprintf(file, "%20.8f,  %20.8f,  %20.8f,  %20.8f\n", 
                      OR_pat[k], se_OR_pat[k], lower, upper);
           }   

           fprintf(file, "\n# Estimates of odds ratios for pre-immunity level\n");
           for(k=0; k<n_imm_covariate; k++)
           {
              lower = exp(coeff_imm[k] - 1.96 * se_coeff_imm[k]);
              upper = exp(coeff_imm[k] + 1.96 * se_coeff_imm[k]);
              fprintf(file, "%20.8f,  %20.8f,  %20.8f,  %20.8f\n", 
                      OR_imm[k], se_OR_imm[k], lower, upper);
           }   

           fprintf(file, "\n# Estimates of CPI\n");
           for(k=0; k<n_b_mode; k++)
           {
              CPI = 1.0 - pow((1.0 - b[k]), cfg_pars.CPI_duration);
              der[0] = cfg_pars.CPI_duration * pow(1.0 - b[k], cfg_pars.CPI_duration - 1.0);
              se_CPI = fabs(der[0]) * se_b[k];
              lower = 1.0 - pow((1.0 - inv_logit(lb[k] - 1.96 * se_lb[k])), cfg_pars.CPI_duration);
              upper = 1.0 - pow((1.0 - inv_logit(lb[k] + 1.96 * se_lb[k])), cfg_pars.CPI_duration);
              fprintf(file, "%e,  %e,  %e,  %e\n", CPI, se_CPI, lower, upper);
           }   
           
           fprintf(file, "\n# Estimates of SAR\n");
           for(k=0; k<n_p_mode; k++)
           {
              n = cfg_pars.effective_upper_infectious[k] - cfg_pars.effective_lower_infectious[k] + 1;
              p_esc = 1.0;
              log_p_esc_der = 0.0;
              for(l=0; l<n; l++)  
              {
                 p_esc *= (1.0 - p[k] * cfg_pars.prob_infectious[l]);
                 log_p_esc_der += -cfg_pars.prob_infectious[l] / (1.0 - p[k] * cfg_pars.prob_infectious[l]);
              }
              SAR = 1.0 - p_esc;
              der[0] = - p_esc * log_p_esc_der;  //der[0] is d(SAR) / d (p[k])
              se_SAR = fabs(der[0]) * se_p[k];
              se_logit_SAR = se_SAR / (SAR * (1 - SAR));
              lower = inv_logit(logit(SAR) - 1.96 * se_logit_SAR);
              upper = inv_logit(logit(SAR) + 1.96 * se_logit_SAR);
              fprintf(file, "%e,  %e,  %e,  %e\n", SAR, se_SAR, lower, upper);
           }   

           fprintf(file, "\n# Estimates of R0\n");
           for(k=0; k<n_p_mode; k++)
           {
              n = cfg_pars.effective_upper_infectious[k] - cfg_pars.effective_lower_infectious[k] + 1;
              p_esc = 1.0;
              log_p_esc_der = 0.0;
              for(l=0; l<n; l++)
              {
                 p_esc *= (1.0 - p[k] * cfg_pars.prob_infectious[l]);
                 log_p_esc_der += -cfg_pars.prob_infectious[l] / (1.0 - p[k] * cfg_pars.prob_infectious[l]);
              }
              SAR = 1.0 - p_esc;
              R0 = cfg_pars.R0_n_contact[k] * SAR;
              der[0] = - cfg_pars.R0_n_contact[k] * p_esc * log_p_esc_der;  //der[k] is d(R0) / d (p[k])
              se_R0 = der[0] * se_p[k];
              lower = exp( log(R0) - 1.96 * se_R0 / R0);
              upper = exp( log(R0) + 1.96 * se_R0 / R0);
              fprintf(file, "%20.8f,  %20.8f,  %20.8f,  %20.8f\n", R0, se_R0, lower, upper);
           }

           fprintf(file, "\n#Covariance Matrix Estimates\n");
           fmfprintf(file, &var, 1);
     
           fclose(file);
        }
     } //if(estimation_error_type == 0)

     //clean structures that are used for each specific epidemic.
     //These structures are created during the estimation
     if(people != NULL)
     {
        for(i=0; i<p_size; i++)
        {
           h = people[i].community;
           if(people[i].possible_status != NULL) 
           {
              free_2d_array_int(people[i].possible_status);
              people[i].possible_status = NULL;
              people[i].size_possible_status = 0;
           }

           if(people[i].risk_history != NULL)
           {
              for(r=0; r<community[h].epi_duration; r++)
              {
                 ptr_risk = people[i].risk_history[r].c2p_risk;
                 while(ptr_risk != NULL)
                 {
                    ptr2_risk = ptr_risk->next;
                    if(ptr_risk->covariate != NULL)  free(ptr_risk->covariate);
                    free(ptr_risk);
                    ptr_risk = ptr2_risk;
                 }
                 ptr_risk = people[i].risk_history[r].p2p_risk;
                 while(ptr_risk != NULL)
                 {
                    ptr2_risk = ptr_risk->next;
                    if(ptr_risk->covariate != NULL)  free(ptr_risk->covariate);
                    free(ptr_risk);
                    ptr_risk = ptr2_risk;
                 }
                 people[i].risk_history[r].c2p_risk = NULL;
                 people[i].risk_history[r].p2p_risk = NULL;
                 person->final_risk_day = MISSING;
              }
           }   
        }
     }   

     if(community != NULL)
     {
        for(h=0; h < n_community; h++)
        {
           if(community[h].member_impute != NULL)  
           {
              free(community[h].member_impute);
              community[h].member_impute = NULL;
              community[h].size_impute = 0;
              community[h].size_possible_status = 1;
           }
           ptr_stat = community[h].sample_status;
           while(ptr_stat != NULL) 
           {
              ptr2_stat = ptr_stat->next;
              free(ptr_stat->status);
              free(ptr_stat);
              ptr_stat = ptr2_stat;
           }    
           community[h].sample_status = NULL;
           community[h].sample_status_rear = NULL;
        }
     }
     free(importance_weight);
     size_sample_status = 0;
  }
  //if(cfg_pars.goodness_of_fit == 1)
  //   goodness_of_fit(est);

end:
  free_arrays();
  deflate_matrix(&var);
  deflate_matrix(&var_logit);
  free(est);
  free(der);
  free(value);
  free(pdf_incubation);
  free(sdf_incubation);

  free(cfg_pars.prob_incubation);
  free(cfg_pars.prob_infectious);
  free(cfg_pars.c2p_covariate);
  free(cfg_pars.sus_p2p_covariate);
  free(cfg_pars.inf_p2p_covariate);
  free(cfg_pars.pat_covariate);
  free(cfg_pars.imm_covariate);
  free_2d_array_int(cfg_pars.interaction);
  free_2d_array_double(cfg_pars.ini_par_effective);
  free(cfg_pars.lower_search_bound);
  free(cfg_pars.upper_search_bound);
  free(cfg_pars.converge_criteria);
  free(cfg_pars.R0_n_contact);
  free(cfg_pars.par_fixed_id);
  free(cfg_pars.par_fixed_value);
  free(cfg_pars.sim_par_effective);
  free(cfg_pars.effective_lower_infectious);
  free(cfg_pars.effective_upper_infectious);
  if(cfg_pars.par_equiclass != NULL)
  {
     for(i=0; i<cfg_pars.n_par_equiclass; i++)
     {
        free(cfg_pars.par_equiclass[i].member);
     }
     free(cfg_pars.par_equiclass);
  }


  if(people != NULL)
  {
     for(i=0; i<p_size; i++)
     {
        h = people[i].community;
        free(people[i].time_ind_covariate);
        free_2d_array_double(people[i].time_dep_covariate);

        if(people[i].pat_covariate != NULL) free_2d_array_double(people[i].pat_covariate);
        if(people[i].imm_covariate != NULL) free(people[i].imm_covariate);
        if(people[i].possible_status != NULL) free_2d_array_int(people[i].possible_status);

        if(people[i].contact_history != NULL)
        {
           for(r=0; r<community[h].epi_duration; r++)
           {
              ptr_contact = people[i].contact_history[r].c2p_contact;
              while(ptr_contact != NULL)
              {
                 ptr2_contact = ptr_contact->next;
                 if(ptr_contact->covariate != NULL)  free(ptr_contact->covariate);
                 free(ptr_contact);
                 ptr_contact = ptr2_contact;
              }
              ptr_contact = people[i].contact_history[r].p2p_contact;
              while(ptr_contact != NULL)
              {
                 ptr2_contact = ptr_contact->next;
                 if(ptr_contact->covariate != NULL)  free(ptr_contact->covariate);
                 free(ptr_contact);
                 ptr_contact = ptr2_contact;
              }
              ptr_risk = people[i].risk_history[r].c2p_risk;
              while(ptr_risk != NULL)
              {
                 ptr2_risk = ptr_risk->next;
                 if(ptr_risk->covariate != NULL)  free(ptr_risk->covariate);
                 free(ptr_risk);
                 ptr_risk = ptr2_risk;
              }
              ptr_risk = people[i].risk_history[r].p2p_risk;
              while(ptr_risk != NULL)
              {
                 ptr2_risk = ptr_risk->next;
                 if(ptr_risk->covariate != NULL)  free(ptr_risk->covariate);
                 free(ptr_risk);
                 ptr_risk = ptr2_risk;
              }
           }
           free(people[i].contact_history);
           free(people[i].risk_history);
        }   
     }
     free(people);
  }   

  if(community != NULL)
  {
     for(h=0; h < n_community; h++)
     {
        if(community[h].member != NULL)  free(community[h].member);
        if(community[h].member_impute != NULL)  free(community[h].member_impute);
        ptr_stat = community[h].sample_status;
        while(ptr_stat != NULL) 
        {
           ptr2_stat = ptr_stat->next;
           free(ptr_stat->status);
           free(ptr_stat);
           ptr_stat = ptr2_stat;
        }    
        ptr_class = community[h].risk_class;
        while(ptr_class != NULL) 
        {
           ptr_integer = ptr_class->member;
           while(ptr_integer != NULL)
           {
              ptr2_integer = ptr_integer->next;
              free(ptr_integer);
              ptr_integer = ptr2_integer;
           }
           ptr2_class = ptr_class->next;
           free(ptr_class);
           ptr_class = ptr2_class;
        }    
     }
     free(community);
  }

  t2 = time(NULL);
  if(cfg_pars.silent_run == 0)  printf("\n use time: %f seconds\n", difftime(t2, t1));
  return(0);
}


