
int simulate()
{
  FILE *file;     /* file pointer, needed to handle i/o to files */
  int loop, id, h, i, j, k, l, m, n, r, t;
  int length, count;
  int n_exp_c, n_inf_c, n_exp_a, n_inf_a;
  int n_imm, n_sym, n_asym, n_esc;
  int b_mode, p_mode, q_mode, u_mode;
  int n_b_mode, n_p_mode, n_q_mode, n_u_mode;
  int n_c2p_covariate, n_p2p_covariate, n_imm_covariate, n_pat_covariate;
  int n_covariate, n_time_ind_covariate, n_time_dep_covariate;
  int n_sus_p2p_covariate, n_inf_p2p_covariate, n_int_p2p_covariate;
  int n_par, n_par_equiclass;

  double prob_esc, prob_imm, prob_sym, s;
  double covariate_effect, temp, *par; 
  CONTACT *ptr_contact;

  n_b_mode = cfg_pars.n_b_mode;
  n_p_mode = cfg_pars.n_p_mode;
  n_u_mode = cfg_pars.n_u_mode;
  n_q_mode = cfg_pars.n_q_mode;
  n_time_ind_covariate = cfg_pars.n_time_ind_covariate;
  n_time_dep_covariate = cfg_pars.n_time_dep_covariate;
  n_covariate = cfg_pars.n_covariate;
  n_c2p_covariate = cfg_pars.n_c2p_covariate;
  n_sus_p2p_covariate = cfg_pars.n_sus_p2p_covariate;
  n_inf_p2p_covariate = cfg_pars.n_inf_p2p_covariate;
  n_int_p2p_covariate = cfg_pars.n_int_p2p_covariate;
  n_p2p_covariate = cfg_pars.n_p2p_covariate;
  n_pat_covariate = cfg_pars.n_pat_covariate;
  n_imm_covariate = cfg_pars.n_imm_covariate;
  n_par = cfg_pars.n_par;
  n_par_equiclass = cfg_pars.n_par_equiclass;

  make_1d_array_double(&par, n_par, 0.0);
  for(i=0; i<n_par_equiclass; i++)
  {
     for(j=0; j<cfg_pars.par_equiclass[i].size; j++)
     {
        m = cfg_pars.par_equiclass[i].member[j] - 1;
        par[m] = cfg_pars.sim_par_effective[i];
     }
  }

  //if fixed parameters exist, need to set fixed parameters to prespecified values
  if(cfg_pars.n_par_fixed > 0)
  {
     for(i=0; i<cfg_pars.n_par_fixed; i++)
     {
        j = cfg_pars.par_fixed_id[i] - 1;
        par[j] =  cfg_pars.par_fixed_value[i];
     }
  }

  for(i=0; i<n_b_mode; i++)  {lb[i] = par[i]; b[i] = inv_logit(lb[i]);}
  for(i=0; i<n_p_mode; i++)  {lp[i] = par[n_b_mode + i]; p[i] = inv_logit(lp[i]);}
  for(i=0; i<n_u_mode; i++)  {lu[i] = par[n_b_mode + n_p_mode + i]; u[i] = inv_logit(lu[i]);}
  for(i=0; i<n_q_mode; i++)  {lq[i] = par[n_b_mode + n_p_mode + n_u_mode + i]; q[i] = inv_logit(lq[i]);}
  for(i=0; i<n_c2p_covariate; i++)  coeff_c2p[i] = par[n_b_mode + n_p_mode + n_u_mode + n_q_mode + i];
  for(i=0; i<n_p2p_covariate; i++)  coeff_p2p[i] = par[n_b_mode + n_p_mode + n_u_mode + n_q_mode
                                                 + n_c2p_covariate + i];
  for(i=0; i<n_pat_covariate; i++)  coeff_pat[i] = par[n_b_mode + n_p_mode + n_u_mode + n_q_mode
                                                 + n_c2p_covariate + n_p2p_covariate + i];
  for(i=0; i<n_imm_covariate; i++)  coeff_imm[i] = par[n_b_mode + n_p_mode + n_u_mode + n_q_mode
                                                 + n_c2p_covariate + n_p2p_covariate + n_pat_covariate + i];

  /* at this point, all is setup to begin the dynamics */
  for(i=0; i < p_size; i++)
  {
     people[i].infection = 0;
     /* Initialize a few things that don't get used until later */
     people[i].day_infection = people[i].day_ill = MISSING;
     people[i].day_infective_lower = people[i].day_infective_upper = MISSING;
     people[i].idx = people[i].pre_immune = people[i].symptom = 0;

     if(n_q_mode > 0)
     {
        covariate_effect = 0.0;
        for(k=0; k<n_imm_covariate; k++)
           covariate_effect += people[i].imm_covariate[k] * coeff_imm[k];
        prob_imm = inv_logit(lq[q_mode] + covariate_effect);
     }
     else  prob_imm = 0.0;
     if(runiform(&seed) <= prob_imm)  people[i].pre_immune = 1;
  }
  
  for(h=0; h<n_community; h++)
  {
     community[h].index_case = -1;
     community[h].idx_day_ill = MISSING;
  }


  /************************************************************
   start the do loop that will run until the epidemic is over
   ************************************************************/
  for(h=0; h<n_community; h++)
  {
     for(t=community[h].day_epi_start; t <= community[h].day_epi_stop; t++)
     {
        for(m=0; m<community[h].size; m++)
        {
           i = community[h].member[m];
           if(people[i].day_infection == MISSING && people[i].pre_immune == 0)
           {
              prob_esc = 1.0;
              r = t - 1;
              ptr_contact = people[i].contact_history[r].c2p_contact;
              while(ptr_contact != NULL)
              {
                 covariate_effect = 0.0;
                 for(k=0; k<n_c2p_covariate; k++)
                    covariate_effect += ptr_contact->covariate[k] * coeff_c2p[k];
                 covariate_effect += ptr_contact->offset;

                 b_mode = ptr_contact->contact_mode;
                 prob_esc *= 1.0 - inv_logit(lb[b_mode] + covariate_effect);
                 ptr_contact = ptr_contact->next;
              }

              ptr_contact = people[i].contact_history[r].p2p_contact;
              while(ptr_contact != NULL)
              {
                 j = ptr_contact->contact_id;
                 if(people[j].day_infection != MISSING && t >= people[j].day_infective_lower 
                                                       && t <= people[j].day_infective_upper )
                 {
                    s = cfg_pars.prob_infectious[t - people[j].day_ill];
                    if( s > 0)
                    {
                       covariate_effect = 0.0;
                       for(k=0; k<n_p2p_covariate; k++)
                          covariate_effect += ptr_contact->covariate[k] * coeff_p2p[k];
                       covariate_effect += ptr_contact->offset;

                       p_mode = ptr_contact->contact_mode;
                       prob_esc *= 1.0 - s * bipow(cfg_pars.asym_effect, 1 - people[j].symptom)
                                 * inv_logit(lp[p_mode] + covariate_effect);
                    }
                 }
                 ptr_contact = ptr_contact->next;
              }    /* if(person->p2p_contact_history[r].size > 0) */

              temp = runiform(&seed);
              if(temp < 1 - prob_esc)
              {
                 people[i].infection = 1;
                 people[i].day_infection = t;
                 if(n_u_mode > 0)
                 {
                    if(u[u_mode] == 0 || u[u_mode] == 1)  prob_sym = u[u_mode];
                    else
                    {
                       covariate_effect = 0.0;
                       for(k=0; k<n_pat_covariate; k++)
                          covariate_effect += people[i].pat_covariate[k][r] * coeff_pat[k];
                       prob_sym = inv_logit(lu[u_mode] + covariate_effect);
                    }
                 }
                 else  prob_sym = 1.0;
                 if(runiform(&seed) < prob_sym)  people[i].symptom = 1;
                 length = cfg_pars.max_incubation - cfg_pars.min_incubation + 1;
                 people[i].day_ill = t + cfg_pars.min_incubation + discrete(length, cfg_pars.prob_incubation, &seed);
                 people[i].day_infective_lower = people[i].day_ill + cfg_pars.lower_infectious;
                 people[i].day_infective_upper = people[i].day_ill + cfg_pars.upper_infectious;
                 //printf("t=%d i=%d h=%d symptom=%d day_ill=%d\n", 
                 //        t, i, people[i].community, people[i].symptom, people[i].day_ill);
                 if(people[i].symptom == 1)
                 {
                    if(community[h].index_case == -1)
                    { 
                       community[h].index_case = i;
                       community[h].idx_day_ill = people[i].day_ill;
                       people[i].idx = 1;
                    }
                    else
                    {
                       if(people[i].day_ill < community[h].idx_day_ill)
                       {
                          j = community[h].index_case;
                          people[j].idx = 0;
                          community[h].index_case = i;
                          community[h].idx_day_ill = people[i].day_ill;
                          people[i].idx = 1;
                       }
                       if(people[i].day_ill == community[h].idx_day_ill)
                       {
                          people[i].idx = 1;
                       }
                    }
                 }
              }
           }
        }
     }
  }  
  
  n_sym = n_asym = n_esc = n_imm = 0;
  for(i=0; i < p_size; i++)
  if(people[i].idx <= 1)
  {
     if(people[i].pre_immune == 1)  n_imm++;
     if(people[i].infection == 0)  n_esc++;
     else
     {
        if(people[i].symptom == 1)  n_sym++;
        else  n_asym++;
     }
  }
  if(cfg_pars.silent_run == 0)  
  {
     printf("before right-censoring: n_imm=%d  n_sym=%d  n_asym==%d  n_esc=%d\n", n_imm, n_sym, n_asym, n_esc);
     printf("preimmune: %d/%d=%e\n", n_imm, p_size, (double) n_imm/p_size);
     printf("attack rate: %d/%d=%e\n", (n_sym+n_asym), p_size - n_imm, (double) (n_sym+n_asym) / (p_size-n_imm));
     printf("pathogenicity: %d/%d=%e\n", n_sym, (n_sym+n_asym), (double) n_sym/(n_sym+n_asym));
  }
  /*
  n_exp_c = n_inf_c = n_exp_a = n_inf_a = 0;
  for(i=0; i < p_size; i++)
  if(people[i].idx <= 1)
  {
     if(people[i].time_ind_covariate[0] == 0)
     {
        n_exp_c++;
        if(people[i].infection == 1)  n_inf_c++;
     }
     else
     {
        n_exp_a++;
        if(people[i].infection == 1)  n_inf_a++;
     }
  }
  printf("children: %d/%d=%e\n", n_inf_c, n_exp_c, (double) n_inf_c/n_exp_c);
  printf("adult: %d/%d=%e\n", n_inf_a, n_exp_a, (double) n_inf_a/n_exp_a);
  
  count = 0;
  for(i=0; i < p_size; i++)
     if(people[i].day_ill != MISSING)  count++;
  printf("Actual total number of cases: %d\n", count);
  */
  //Some cases (symptomatic or asymptomatic) are infected right before the last day but infectiousness onsets extend over the observation period.
  //We set these people to escapes, and this right-censoring of symptoms can be adjusted for in analysis.
  count = 0;
  for(i=0; i < p_size; i++)
  {
     h = people[i].community;
     if(people[i].infection == 1 && people[i].day_ill > community[h].day_epi_stop)
     {
        people[i].infection = 0;
        people[i].day_ill = MISSING;
        people[i].symptom = 0;
     }  
     if(people[i].infection == 1 && people[i].symptom == 0)
     {
        //people[i].day_ill = MISSING;
     }

     if(people[i].day_ill != MISSING) count++;
  }
  
  n_sym = n_asym = n_esc = n_imm = 0;
  for(i=0; i < p_size; i++)
  if(people[i].idx <= 1)
  {
     if(people[i].pre_immune == 1)  n_imm++;
     if(people[i].infection == 0)  n_esc++;
     else
     {
        if(people[i].symptom == 1)  n_sym++;
        else  n_asym++;
     }
  }
  if(cfg_pars.silent_run == 0)  
  {
     printf("After right censoring: n_imm=%d  n_sym=%d  n_asym==%d  n_esc=%d\n", n_imm, n_sym, n_asym, n_esc);
     printf("preimmune: %d/%d=%e\n", n_imm, p_size, (double) n_imm/p_size);
     printf("attack rate: %d/%d=%e\n", (n_sym+n_asym), p_size - n_imm, (double) (n_sym+n_asym) / (p_size-n_imm));
     printf("pathogenicity: %d/%d=%e\n", n_sym, (n_sym+n_asym), (double) n_sym/(n_sym+n_asym));
  }
  free(par);
  return(0);
}




